/*
estimation.h
Copyright (C) yyyy Ruslan Soldatov

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

Contact e-mail: ruslansoldatov@gmail.com
*/

#include "approx_sample.h"
#include "../rnaslider/rnaSlider.h"
#include "cmdline_approx_sample.h"
#include "estimation.h"

int convert_probing(char nt);
int len_probing(FILE* f);
char* read_seq(FILE* file);
double* read_data(FILE* file,int length);

void generate_boundaries(int n, int winmin, int winmax, int sample_size, double param_p, struct energy_sample* smpl){
	double q,cqw,unif;
	int i,gen_len;
	
	q=exp( log(param_p)/(2*n/3 - winmin) );
	cqw=(1-q)/(1-exp((winmax-winmin+1)*log(q)));
	
	for (i=0;i<sample_size;i++){
		unif=((double) rand() / (RAND_MAX+1.0));
		gen_len=(int)( log(1-unif*(1-q)/cqw)/log(q)+winmin-1 ); 
		smpl[i].length=gen_len;
		smpl[i].start= ( rand() % (n-gen_len) ) + 1;
		smpl[i].end=smpl[i].start+gen_len-1;
	}
}

/*
	seq - input sequence
	pair/unpair - structure-probing data, if available
	n - length of the sequence
	winmin - minimum window
	winmax - maximum window
	sample_size - number of subsequence to sample
	param_p - input parameter for sampling
*/
void sample_energies(char* seq, double* pair, double* unpair, int n, int winmin, int winmax, int sample_size, double param_p, struct energy_sample* smpl, char* argv){
	char* subseq;
	int i,j;
	
	srand(time(NULL));
	generate_boundaries(n,winmin,winmax,sample_size,param_p,smpl); // generate boundaries of subsequences;
	for (i=0;i<sample_size;i++){
		subseq=(char*)malloc( (smpl[i].length+1)*sizeof(char));
		strncpy(subseq, seq+(smpl[i].start-1), smpl[i].length);
		subseq[smpl[i].length]='\0';
		smpl[i].energy=runSlider( subseq,NULL,NULL,smpl[i].length,smpl[i].length,0.9,argv); 
		smpl[i].energy_probing=smpl[i].energy;
		if (pair!=NULL & unpair!=NULL){ 
			smpl[i].energy_probing=runSlider( subseq,pair+(smpl[i].start-1),unpair+(smpl[i].start-1),smpl[i].length,smpl[i].length,0.9,argv);
		}
		smpl[i].probed_bases=0;
		if (pair!=NULL & unpair!=NULL){
			for(j=0;j<smpl[i].length;j++)
				if ( pair[j+smpl[i].start-1]!=-1 & unpair[j+smpl[i].start-1]!=-1   )  smpl[i].probed_bases++;
			smpl[i].probed_bases/=smpl[i].length;
		}
		free(subseq);
	}

}

void sample_energies_general(char* seqname, char* pairname, char* unpairname, int winmin, int winmax, int sample_size, double param_p, struct energy_sample* smpl, char* argv){
	double *pair,*unpair;
	char *seq;
	FILE *inp_paired,*inp_unpaired,*inp_seq;
	int n;
	
	if ( (inp_seq = fopen(seqname,"r" )) ==NULL )
	{
		fprintf(stderr,"Error in opening file with sequence\n");
		exit( 1 );
	}
	if ( (inp_paired = fopen(pairname, "r" )) ==NULL )
	{
		fprintf(stderr,"Error in opening file with paired probabilities\n");
		exit( 1 );
	}
		if ( (inp_unpaired = fopen(unpairname, "r" )) ==NULL )
	{
		fprintf(stderr,"Error in opening file with unpaired probabilities\n");
		exit( 1 );
	}
	
	seq=NULL;pair=NULL;unpair==NULL;
	if (inp_seq!=NULL)
		seq=read_seq(inp_seq);
	if (inp_paired!=NULL)
		pair=read_data(inp_paired,strlen(seq));
	if (inp_unpaired!=NULL)
		unpair=read_data(inp_unpaired,strlen(seq));
	
	n=strlen(seq);
	sample_energies(seq, pair, unpair, n,winmin, winmax, sample_size, param_p, smpl, argv);
	
	fclose(inp_seq);
	fclose(inp_paired);
	fclose(inp_unpaired);
}

/* convert nucleotides to numbers (convert) and back (revert)
	A - 0
	T - 1
	G - 2
	C - 3
*/
int convert_probing(char nt){
	if (nt=='A' || nt=='a') 
		return 0;
	else if (nt=='T' || nt=='t' || nt=='U' || nt=='u')
		return 1;
	else if (nt=='G' || nt=='g')
		return 2;
	else if (nt=='C' || nt=='c') 
		return 3;
	else{
		//fprintf(stderr,"Unknown character: %c\n",nt);
		return -1;
	}
}

/* return sequence length */
int len_probing(FILE* f)
{
	int i=0;
	char c;
	while ( feof(f)==0 )
	{
		c=fgetc(f);
		if ( convert_probing(c)!=-1 ) i++;
		else if ( c!='\n' && c!='\r' && c!=' ' && c!='\t' && c!=-1 ) i++;
	}
	return i;
}


char* read_seq(FILE* file){
	char c;
	char* tmp;
	int length,i;
	fpos_t f_pos;
	
	if ( (c=fgetc(file)) != '>' ){
		fprintf(stderr,"Incorrect fasta-file");
		exit(1);
	}
	while ( (c=fgetc(file)) != '\n' && c!='\r' )
		;
	fgetpos(file,&f_pos);
	length=len_probing(file);
	fsetpos(file,&f_pos);
	tmp=(char*)malloc( (length+1)*sizeof(char));
	i=0;
	while ( feof(file)==0 ){
		c=fgetc(file);
		if ( convert_probing(c)!=-1 || (c!='\n' && c!='\r' && c!=' ' && c!='\t' && c!=-1) ){
			tmp[i]=c;
			i++;
		}
	}
	tmp[i]='\0';
	return tmp;
}


double* read_data(FILE* file,int length){
	double* tmp;
	int i;
	double x;
	tmp=(double*)malloc(length*sizeof(double));
	for (i=0;i<length;i++)
		tmp[i]=-1;
	while ( feof(file)==0 ){
		fscanf(file,"%d %lf",&i,&x);
		tmp[i-1]=x;
	}
	return tmp;
}


int estimateRegPar(struct energy_sample* smpl,int sample_size,double *a, double *b){
	double x,y,x2,xy;
	int i;
	
	x=0;
	y=0;
	x2=0;
	xy=0;
	for (i=0;i<sample_size;i++){
		x+=smpl[i].energy;
		y+=smpl[i].energy_probing;
		x2+=smpl[i].energy*smpl[i].energy;
		xy+=smpl[i].energy*smpl[i].energy_probing;
	}
	*a = (xy-x*y/sample_size)/(x2-x*x/sample_size);
	*b = (y-(*a)*x)/sample_size;

	return 0;
}

/*
int main(int argc, char* argv[]){
	char *iname_seq,*iname_paired,*iname_unpaired,*outname;
	FILE *inp_seq,*inp_paired,*inp_unpaired,*output;
	char* seq;
	int i, n, winmin=20, winmax=300;
	int sample_size=100;
	double param_p=0.01;
	double a,b;
	struct energy_sample* smpl;
	double *paired=NULL,*unpaired=NULL;
	struct gengetopt_args_info args_info;

	if (cmdline_parser(argc, argv, &args_info) != 0)
    {
		fprintf(stderr,"Wrong command line options\n");
		exit(1);
	}
	if ( args_info.input_given )
		iname_seq=args_info.input_arg;
	if ( args_info.paired_given )
		iname_paired=args_info.paired_arg;
	if ( args_info.unpaired_given )
		iname_unpaired=args_info.unpaired_arg;
	if ( args_info.output_given )
		outname=args_info.output_arg;
	if ( args_info.winmax_given )
		winmax=args_info.winmax_arg;
	if ( args_info.winmin_given )
		winmin=args_info.winmin_arg;
	if ( args_info.size_given)
		sample_size=args_info.size_arg;	
	if ( args_info.par_given )
		param_p=args_info.par_arg;	
	
	if ( (inp_seq = fopen(iname_seq, "r" )) == NULL ){
		fprintf(stderr,"Error in opening input file with sequence for read\n");
		exit( 1 );
	}	
	if ( args_info.paired_given & (inp_paired = fopen(iname_paired, "r" )) == NULL )
	{
		fprintf(stderr,"Error in opening input file with paired probabilities for read\n");
		exit( 1 );
	}	
	if ( args_info.unpaired_given & (inp_unpaired = fopen(iname_unpaired, "r" )) == NULL )
	{
		fprintf(stderr,"Error in opening input file with unpaired probabilities for read\n");
		exit( 1 );
	}
	if ( args_info.output_given & (output = fopen(outname, "w" )) == NULL )
	{
		fprintf(stderr,"Error in opening input file for write\n");
		exit( 1 );
	}	
	
	/* read sequence and data */
	/*
	seq=read_seq(inp_seq);
	if ( args_info.paired_given )
		paired=read_data(inp_paired,strlen(seq));
	if ( args_info.unpaired_given )
		unpaired=read_data(inp_unpaired,strlen(seq));
		
	if ( winmax > strlen(seq) )
		winmax=strlen(seq);
	if ( winmin > strlen(seq) )
		winmin=strlen(seq);
		
	if ( !args_info.output_given )
		output=stdout;
	
	
	
	n=strlen(seq);
	smpl=(struct energy_sample*)malloc( sample_size*sizeof(struct energy_sample));
	sample_energies(seq, paired, unpaired, n, winmin, winmax, sample_size, param_p, smpl,argv[0]);
	//sample_energies_general( iname_seq, iname_paired, iname_unpaired, winmin, winmax, sample_size, param_p, smpl, argv[0]);

	
	estimateRegPar(smpl,sample_size,&a, &b);
	printf("a: %lf, b: %lf\n",a,b);
	for (i=0;i<sample_size;i++)
		fprintf(output,"%d %d %d %lf %lf %lf\n",smpl[i].start,smpl[i].end,smpl[i].length,smpl[i].energy,smpl[i].energy_probing,smpl[i].probed_bases);
			
		
	if ( !args_info.output_given )
		fclose(output);
	if ( args_info.paired_given )
		fclose(inp_paired);
	if ( args_info.unpaired_given )
		fclose(inp_unpaired);
	if ( args_info.input_given )
		fclose(inp_seq);
		
	return 0;
	
}

*/