/*
fileproc.c
Copyright (C) yyyy Ruslan Soldatov

* Fast and accurate identification of locally optimal motfis with high structural potential

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

Contact e-mail: ruslansoldatov@gmail.com
*/
#include "rnaSurface.h"
#include <sys/mman.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#define handle_error(msg) \
do { perror(msg); exit(EXIT_FAILURE); } while (0)

#define mmap_size /*1000000;*/200000000;

struct unit
{
	int pos;
	float value;
};

struct motifLO
{
	int start,end;
	float energy;
	float z;
};

/*
	Description: convert binary file to text file
	option: 0 - print sign file, 1 - print density file
*/
void fileConverter(char* chr,FILE* inputBin,FILE* outputText,int option)
{
	int i;
	float x;
	
	fseek (inputBin,0,SEEK_SET);
	fprintf(outputText,"track\ttype=wiggle_0\n");
	fprintf(outputText,"fixedStep\tchrom=%s\tstart=1\tstep=1\n",chr);
	while ( fread(&i,sizeof(int),1,inputBin)!=0 )
	{
		fread(&x,sizeof(float),1,inputBin);
		if (option==0)
		{
			if (x>0)	x=0;
			else x=x*x;
		}
		fprintf(outputText,"%f\n",x);
	}
}

/*
	Description: merge value of two strands
	nameplus - name of file with plus strand
	nameminus - name of file with minus strand
	option: if 0 - merge sign files, if 1 - merge density files
	
*/
void mergeStrands(char* nameplus,char* nameminus,int seq_length, int option)
{
	int fdplus,fdminus;
	int elementSize,psize,size,sizeC,i,j,skipBytes,skipBytesC,skip,skipC,len,lenC,pos,count;
	struct unit *addrplus,*addrminus;
	
	fdplus=open(nameplus,O_RDWR | O_CREAT | O_APPEND , S_IRUSR | S_IWUSR);
	if (fdplus == -1)
         handle_error("open");	
	
	fdminus=open(nameminus,O_RDWR | O_CREAT | O_APPEND , S_IRUSR | S_IWUSR);
	if (fdminus == -1)
         handle_error("open");	
		 
	elementSize=sizeof(struct unit);
	psize=getpagesize();
	size=mmap_size;
	size=size-size % psize;
	if ( size > seq_length*elementSize ) size=seq_length*elementSize;

	skipBytes=0;
	skip=0;
	skipBytesC=seq_length*elementSize-(seq_length*elementSize)%size-size;
	skipC=skipBytesC/elementSize;
	sizeC=size+(seq_length*elementSize)%size;
	len=size/elementSize;
	lenC=sizeC/elementSize;
	
	addrplus = mmap(0,size, PROT_READ | PROT_WRITE,MAP_SHARED, fdplus, skipBytes); 
	if (addrplus == MAP_FAILED)
		 handle_error("mmap"); 

	addrminus = mmap(0,sizeC, PROT_READ | PROT_WRITE,MAP_SHARED, fdminus, skipBytesC); 	
	if (addrminus == MAP_FAILED)
		 handle_error("mmap");

	
	count=0;
	i=0;
	while ( count < seq_length )
	{

		if (i==len)
		{
			skipBytes+=size;
			skip+=size/elementSize;
			munmap(addrplus,size);
			
			addrplus = mmap(0,size, PROT_READ | PROT_WRITE,MAP_SHARED, fdplus, skipBytes); 
			if (addrplus == MAP_FAILED)	handle_error("mmap");
			i=0;
			len=size/elementSize;
		}
		if (  seq_length-1-skip-skipC-i < 0 || seq_length-1-skip-skipC-i >= lenC  )
		{
			skipBytesC-=size;
			skipC-=size/elementSize;
			munmap(addrminus,sizeC);
			addrminus = mmap(0,size, PROT_READ | PROT_WRITE,MAP_SHARED, fdminus, skipBytesC); 	
			if (addrminus == MAP_FAILED)handle_error("mmap");
			sizeC=size;
			lenC=sizeC/elementSize;
		}
		{
			j=seq_length-1-skip-skipC-i;
			if (option==0)
			{
				if (addrplus[i].value > addrminus[/*len-1-i*/j].value) addrplus[i].value=addrminus[/*len-1-i*/j].value;
			}
			else if (option==1) 
			{
				addrplus[i].value+=addrminus[/*len-1-i*/j].value;
			}
			i++;
		}
		
		count++;
	}
	
	munmap(addrplus,size);
	munmap(addrminus,size);
	close(fdplus);
	close(fdminus);
}

/*
	Description: merge map files of two strands
	nameplus - name of file with plus strand
	nameminus - name of file with minus strand
*/
void mergeStrandsMap(char* nameplus,char* nameminus,int seq_length, int winWidth)
{
	int fdplus,fdminus;
	int elementSize,psize,size,sizeC,i,skipBytes,skipBytesC,pos,j;
	int posX,posY;
	char *addrplus,*addrminus;

	fdplus=open(nameplus,O_RDWR | O_CREAT | O_APPEND , S_IRUSR | S_IWUSR);
	if (fdplus == -1)
         handle_error("open");	
	
	fdminus=open(nameminus,O_RDWR | O_CREAT | O_APPEND , S_IRUSR | S_IWUSR);
	if (fdminus == -1)
         handle_error("open");	
		 
	psize=getpagesize();
	size=mmap_size;
	size=size-size % psize;
	if ( size > seq_length*winWidth ) size=seq_length*winWidth;
	
	skipBytes=0;
	skipBytesC=seq_length*winWidth-(seq_length*winWidth)%size-size;
	sizeC=size+(seq_length*winWidth)%size;
	
	addrplus = mmap(0,size, PROT_READ | PROT_WRITE,MAP_SHARED, fdplus, skipBytes); 
	if (addrplus == MAP_FAILED)
		 handle_error("mmap"); 
	
	addrminus = mmap(0,sizeC, PROT_READ | PROT_WRITE,MAP_SHARED, fdminus, skipBytesC); 	
	if (addrminus == MAP_FAILED)
		 handle_error("mmap");
	
	posX=0;
	i=0;
	while ( posX < seq_length*winWidth )
	{
		posY=(seq_length-1)*winWidth-posX+2*(posX%winWidth)+1;//+2
		if (i==size)
		{
			skipBytes+=size;
			munmap(addrplus,size);			
			addrplus = mmap(0,size, PROT_READ | PROT_WRITE,MAP_SHARED, fdplus, skipBytes); 
			if (addrplus == MAP_FAILED)	handle_error("mmap");
			i=0;
		}
		
		j=posY-skipBytesC-1;

		if (  (j<2*winWidth && j>=winWidth) && (posX%winWidth == winWidth-1) )
		{
			if (skipBytesC<size-psize) skipBytesC=0;
			else skipBytesC-=size-psize;
			munmap(addrminus,sizeC);
			/*
			printf("---------------------------");
			printf("info: j=%d posX=%d posY=%d\n",j,posX,posY);
			printf("skipBytesC: %d, size+skipBytesC: %d, total: %d",skipBytesC,size+skipBytesC,seq_length*winWidth);
			*/
			addrminus = mmap(0,size, PROT_READ | PROT_WRITE,MAP_SHARED, fdminus, skipBytesC); 	
			if (addrminus == MAP_FAILED)handle_error("mmap");
			sizeC=size;
		}
		
		
		//j=seq_length*winWidth-1-skipBytes-skipBytesC-i+2*(i%winWidth);
		
		/*if ( ( addrplus[i] > addrminus[j] || addrplus[i]<0 ) && ((int)addrminus[j] >= 0) ) 
			addrplus[i]=addrminus[j];
		else if ( ( addrplus[i] > addrminus[j] ) && ((int)addrminus[j] < 0) ) 
			addrplus[i]=addrminus[j];
		*/
		
		if ( (int)addrminus[j] >= 0 && (int)addrplus[i] >= 0 ) 
		{
			if ( (int)addrminus[j] < (int)addrplus[i] ) addrplus[i]=addrminus[j];
		}
		else if ( (int)addrplus[i] < 0 && (int)addrminus[j] >= 0  )  addrplus[i]=addrminus[j];
		else if ( (int)addrminus[j] < 0 && (int)addrplus[i] < 0 )
		{
			if ( (int)addrminus[j] < (int)addrplus[i] ) addrplus[i]=addrminus[j];
		}
				
		//addrplus[i]=addrminus[j];
		i++;	
		posX++;
	}
	
	munmap(addrplus,size);
	munmap(addrminus,size);
	close(fdplus);
	close(fdminus);
}

/*
	Description: Make file of locally optimal structures in bed format using binary files from one or both strands.
	nameplus - locally optimal structures on "+" strand
	nameminus - locally optimal structures on "-" strand
	chr - name of chromosome
	option: 0 - if only one strand used, 1 - otherwise
*/
void makeBedLocOpt(char* nameplus,char* nameminus,FILE *out,char* chr,int seq_length,int option)
{
	struct motifLO motif;
	if (option==0)
	{
		FILE *locOptBinary;
		if ( (locOptBinary=fopen(nameplus,"rb"))==NULL )
		{
			fprintf(stderr,"Cannot open binary file with locally optimal structrues for converting to bed-format");
			exit(1);
		}
		
		while ( fread(&motif,sizeof(struct motifLO),1,locOptBinary) != 0 ) 
		{
			fprintf(out,"%s\t%d\t%d\tE=%f\t%f\t+\n",chr,motif.start-1,motif.end,motif.energy,motif.z);
		}
		fclose(locOptBinary);
	}
	else if (option==1)
	{
		int fdplus,fdminus;
		int elementSize,psize,size,sizeC,skipBytes,skipBytesC,skip,skipC;
		int ind,indC;
		struct stat splus,sminus;
		struct motifLO plusLO,minusLO;
		struct motifLO *addrplus,*addrminus;
		//printf("In function\n");
		
		
		fdplus=open(nameplus,O_RDWR | O_CREAT | O_APPEND , S_IRUSR | S_IWUSR);
		if (fdplus == -1) handle_error("open");	
	
		fdminus=open(nameminus,O_RDWR | O_CREAT | O_APPEND , S_IRUSR | S_IWUSR);
		if (fdminus == -1) handle_error("open");	
			
		if (fstat(fdplus, &splus) == -1) handle_error("fstat");
		if (fstat(fdminus, &sminus) == -1) handle_error("fstat");
		
		elementSize=sizeof(struct motifLO);
		psize=getpagesize();
		size=mmap_size;
		size=size-size % psize;
		if ( size > splus.st_size ) size=splus.st_size;
		sizeC=size+(sminus.st_size)%size;
		if ( sizeC > sminus.st_size ) sizeC=sminus.st_size;
		
		skipBytes=0;
		skip=0;
		skipBytesC=sminus.st_size-sizeC;
		skipC=skipBytesC/elementSize;
	
		addrplus = mmap(0,size, PROT_READ | PROT_WRITE,MAP_SHARED, fdplus, skipBytes); 
		if (addrplus == MAP_FAILED)
			handle_error("mmap"); 

		addrminus = mmap(0,sizeC, PROT_READ | PROT_WRITE,MAP_SHARED, fdminus, skipBytesC); 	
		if (addrminus == MAP_FAILED)
			handle_error("mmap");
	
		ind=0;
		indC=sizeC/elementSize-1;
		
		while( ( skipBytes+ind*elementSize < splus.st_size ) || ( skipBytesC+indC*elementSize >= 0 )  )
		{
			//printf("%d %d %d\n", skipBytes+ind*elementSize , splus.st_size , skipBytesC+indC*elementSize );
			//printf("%d %d %d\n", ind*elementSize , size , indC );
			if ( ind*elementSize >= size && ( skipBytes+ind*elementSize < splus.st_size ) )
			{
				//printf("here1\n");
				skipBytes+=size;
				munmap(addrplus,size);			
				addrplus = mmap(0,size, PROT_READ | PROT_WRITE,MAP_SHARED, fdplus, skipBytes); 
				if (addrplus == MAP_FAILED)	handle_error("mmap");
				ind=0;
			}
			if (indC == -1 && ( skipBytesC+indC*elementSize >= 0 ) )
			{
				//printf("here2\n");
				skipBytesC-=size;
				munmap(addrminus,sizeC);
				addrminus = mmap(0,size, PROT_READ | PROT_WRITE,MAP_SHARED, fdminus, skipBytesC); 	
				if (addrminus == MAP_FAILED)handle_error("mmap");
				sizeC=size;
				indC=sizeC/elementSize-1;
			}
			
			//printf("%d %d %d %d\n", skipBytes+ind*elementSize - splus.st_size , skipBytesC+indC*elementSize , indC , sizeC % elementSize );
			
			if ( ( skipBytes+ind*elementSize < splus.st_size ) && ( skipBytesC+indC*elementSize < 0 )  )
			{
				fprintf(out,"%s\t%d\t%d\tE=%f\t%f\t+\n",chr,addrplus[ind].start-1,addrplus[ind].end,addrplus[ind].energy,addrplus[ind].z);
				ind++;
			}
			else if ( ( skipBytes+ind*elementSize >= splus.st_size ) && ( skipBytesC+indC*elementSize >= 0 )  )
			{
				fprintf(out,"%s\t%d\t%d\tE=%f\t%f\t-\n",chr,seq_length-addrminus[indC].end+1-1,seq_length-addrminus[indC].start+1,addrminus[indC].energy,addrminus[indC].z);
				indC--;
			}
			else 
			{
				
				plusLO.start=addrplus[ind].start;plusLO.end=addrplus[ind].end;plusLO.energy=addrplus[ind].energy;plusLO.z=addrplus[ind].z;
							
				minusLO.start=seq_length-addrminus[indC].end+1;minusLO.end=seq_length-addrminus[indC].start+1;minusLO.energy=addrminus[indC].energy;minusLO.z=addrminus[indC].z;
				
				//printf("pos: %d %d %d %d\n",ind,indC,plusLO.start,minusLO.start);
				
				if ( plusLO.start < minusLO.start )
				{
					fprintf(out,"%s\t%d\t%d\tE=%f\t%f\t+\n",chr,plusLO.start-1,plusLO.end,plusLO.energy,plusLO.z);
					ind++;
				}
				else if ( plusLO.start > minusLO.start )
				{
					fprintf(out,"%s\t%d\t%d\tE=%f\t%f\t-\n",chr,minusLO.start-1,minusLO.end,minusLO.energy,minusLO.z);
					indC--;
				}
				else
				{
					if ( plusLO.end < minusLO.end )
					{
						fprintf(out,"%s\t%d\t%d\tE=%f\t%f\t+\n",chr,plusLO.start-1,plusLO.end,plusLO.energy,plusLO.z);
						ind++;
					}
					else
					{
						fprintf(out,"%s\t%d\t%d\tE=%f\t%f\t-\n",chr,minusLO.start-1,minusLO.end,minusLO.energy,minusLO.z);
						indC--;
					}
				}
			}
			
		}
			
		munmap(addrplus,size);
		munmap(addrminus,sizeC);
		close(fdplus);
		close(fdminus);		
	}
}