/*
rnaSurface.c
Copyright (C) yyyy Ruslan Soldatov

* RNASurface: fast and accurate detection of locally optimal potentially structured RNA segments

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

Contact e-mail: ruslansoldatov@gmail.com
*/
#include "rnaslider/rnaSlider.h"
#include "rnaslider/common.h"
#include "rnaslider/energyfiles/energy.h"
#include "rnaslider/energyfiles/energy_read.h"
#include "coef.h"
#include "rnaSurface.h"
#include "cmdline.h"
#include "estimation/approx_sample.h"

/* Z-score matrix */
float** Zscore;
/* Z-score extension */
float** Zbig;
/* array of minimum Z-score in each position */
float* minScore;
/* length of dinucleotide content of sequence */
int d;
/* array of dinucleotides in different windows */
int** di;
/* means of control lengths for different dinucleotide composition */
double** control_mean;
/* standart deviations of control length for different dinucleotide composition */
double** control_sd;
/* array of nucleotides counts */
int** nucl;
/* current subsequence */
char* str;
/* array of regression numbers fo each new window */
int* regressionNum;
/* length of input sequence*/
int len_inpseq;
/* length of arrays */
int len_par;
/* length of 'str' sequence */
int len_str;
/* arrays of start and end of energy window for sequence */
int* startWin;
int* endWin;
/* array of start and end of dinucleotide window for sequence */
int startPoint;
/* quantity of undefined points */
int* undefPoint;
/* min length */
//int winMin;
/* name of chromosome*/
char* chrName;
/* whether use energy expectation and sd from file or tabulated values */
int useParams;
/* whether to search locally optimal accessiblity sites */
int accessible;
double slope_E_cov, slope_E, intercept;

/* matrix of smooth Z-scores */
float** Zsmooth;
long double *SL,*SR,*B1,*B2,*B3,*B4;
/* matrixes of factors */
float* factors;
long double* qp;
/* parameter of smooth*/
int smooth_param;
int zdigits,qdigits;
/* calculate smoothed Z-score surface */
void make_smooth(int n_smooth,int option);
float calculate_smooth(int i,int j,int n_smooth);
float smooth(int i,int j);
long double qpow(int qq,int k);
long double nfloat(float e,int n);
void step(int pos_j);

/* calculate Z-score surface */
void zSurface(FILE* inp,FILE* outp,double Zthre,int smooth,int local,int print_current, char* vtimeFilename,int vtime,int timeNum, int print_map,FILE* mapFile,FILE* mapFileKeys, int print_minScore,FILE* minscoreFile);
/* initialize arrays di,control_mean,control_sd,nucl,str,fin_ind */
void initializeParam(int n_smooth,int f_length);
/* calculate indent to the left for determination dinucleotide composition for sequence of length l */
int indent(int l);
/* print z-score to Z array */
void initializeSurface(int n_smooth, double Zthre);
/* return integer representation of nucleotide*/
int nuclNum(char c);
/* return regression number*/
int regressionNumber(int na,int nt,int ng,int nc);
/* calculcate parameters for last the new nucleotide*/
void recalculate_param(char c);
/* calculate Zscores for new element*/
void extendZscore(int n_smooth);
/* define and print local optimal structures*/
void printLocOpt(FILE* out,int n_locOpt,double Zthre,int option);
/* calculate minimum Z-score in position */
void calculateMinScore(int option, double Zthre);
/* print minimum Z-score in position */
void printMinScore(FILE* out, int opt);
/* print Z-score surface */
void printMap(FILE* out,int option);
/* determine length of next sequence from file */
int len(FILE* f);
/* convert z-score to byte representation */
char convert(double z);
/* determine length of sequence from file */
int seqFileLength(char* filename);
/* return fraction of experimentally covered bases of subsequence [i..j]*/
double cover(int i, int j);
int nq;
/* count time*/
double start_t,end_t;


void free_param();
float E(int i,int j);
float Z(int i,int j);
float ZbigScore(int i,int j);

/*  
	Description: function realize algorithm
	
	in - input file with sequences for read (if read from file)
	out - output file for locally optimal structures write
	seqin - sequence from stdin (if have)
	seq_name - name of sequence from stdin (if have)
	option - 0 for reading from file, 1 for reading from stdin
	Zthre - z threshold
	n_smooth - smooth parameter
	n_locOpt - locOpt parameter
	print_current - 1 if print every 'vstep' nucleotides, 0 otherwise
	vtimeFilename - filename to print process state
	vstep - print state every vstep nucleotides
	timeNum - forward(1)/reverse(0) strand
	print_map - 1 if print heatmap values, 0 otherwise
	mapFile - heatmap file for writing
	print_minScore - 1 if print minScore values, 0 otherwise
	minscoreFile - file with minimum scores for writing
*/


void zSurface(FILE* in,FILE* out,double Zthre,int n_smooth,int n_locOpt,int print_current,char* vtimeFilename,int vstep,int timeNum,int print_map,FILE* mapFile,FILE* mapFileKeys, int print_minScore,FILE* minscoreFile)
{
	char c,cfirst;
	int i,j,pos1,pos2,wp1,f_length;
	double seq_process,time,time_left;
	fpos_t f_pos;
	char name[256];
	fscanf(in,"%s",name);
	if (name[0]!='>')
	{
		fprintf(stderr,"Input sequence must be in fasta format");
		exit(16);
	}

	while ( (c=fgetc(in))!='\n' && c!='\r'  )
		;
	
	fgetpos(in,&f_pos);
	f_length=len(in);
	len_inpseq=f_length;
	fsetpos(in,&f_pos);
	
	/*** check parameters ***/
	if (d>f_length) d=f_length;
	if (winSize>f_length)
	{
		winSize=f_length;
		//fprintf(stderr,"Sequence smaller than window size");
		//exit(17);
	}
	if (print_map==1 && mapFileKeys!=NULL){
		fprintf(mapFileKeys,"%d\n%d",(winSize-winMin+1),(winMin-1));
		fclose(mapFileKeys);
	}
	/******/
	/*** initialize varablies and start calculations ***/
	define_bounds(winMin,winSize);
	initializeParam(n_smooth,f_length);
	i=0;
	while ( (i<len_str) && feof(in)==0 && (c=fgetc(in))!='>' )
	{
		//c=fgetc(in);
		if (nuclNum(c)!=-1)
		{
			str[i]=c;
			i++;
		}
	}
	for (i=0;i<winSize;i++) undefPoint[i]=0;
	winPoint=0;startPoint=0;
	optEnergy(str, winSize);

	initializeSurface(n_smooth,Zthre);
	//printf("zscore %d %d %lf\n",1,winSize,Zscore[0][winSize-1]);		
	if (print_minScore==1)printMinScore(minscoreFile,0);
	if (print_map==1)printMap(mapFile,1);
	winPoint=1;

	while ( /*feof(in)==0  &&*/  winPoint+winSize+indent(winSize)<d )
	{
		extendOptEnergy(str+(winPoint-startPoint), winSize);
		recalculate_param(cfirst);
		extendZscore(n_smooth);
		calculateMinScore(0,Zthre);
		if (winPoint>=smooth_param) printLocOpt(out,n_locOpt,Zthre,0);
		if (print_minScore==1)printMinScore(minscoreFile,0);
		if (print_map==1)printMap(mapFile,0);
		winPoint++;
	}
	/******/
	while ( feof(in)==0 && (c=fgetc(in))!='>' )
	{
		/*** next symbol is nucleotide ***/
		if ( nuclNum(c) != -1 )
		{
			for (i=0;i<winSize-1;i++) undefPoint[i]=undefPoint[i+1];	
			cfirst=str[0];
			for (i=0;i<len_str-1;i++)str[i]=str[i+1];
			str[len_str-1]=c;
			
			extendOptEnergy(str+(winPoint-startPoint), winSize);
			recalculate_param(cfirst);
			extendZscore(n_smooth);
			calculateMinScore(0,Zthre);
			if (winPoint>=smooth_param) printLocOpt(out,n_locOpt,Zthre,0);
			if (print_minScore==1)printMinScore(minscoreFile,0);
			if (print_map==1)printMap(mapFile,0);
			if (print_current==1 && winPoint%vstep==0) 
			{
				seq_process=(double)winPoint/f_length*100;
				if (timeNum==1) seq_process=seq_process/2;
				else if (timeNum==2) seq_process=50+seq_process/2;
				end_t=clock();
				time=(end_t-start_t)/CLOCKS_PER_SEC;
				time_left=(100-seq_process)/seq_process*time;
				printf("--------------------\n");
				printf("sequence process: %d%%\n",(int)seq_process);
				printf("time left: %d sec\n",(int)time_left);
			}
			if (vtimeFilename != NULL && winPoint%vstep==0)
			{
				FILE* vtimeFile;
				vtimeFile=fopen(vtimeFilename,"w");
				if ( vtimeFile != NULL)
				{
					seq_process=(double)winPoint/f_length*100;
					if (timeNum==1) seq_process=seq_process/2;
					else if (timeNum==2) seq_process=50+seq_process/2;
					end_t=clock();
					time=(end_t-start_t)/CLOCKS_PER_SEC;
					time_left=(100-seq_process)/seq_process*time;
					fprintf(vtimeFile,"sequence process: %d%%\n",(int)seq_process);
					fprintf(vtimeFile,"time left: %d sec\n",(int)time_left);
				}
				fclose(vtimeFile);
			}
			winPoint++;
			startPoint++;
		}
		/*** unknown character, just skip ***/
		else if ( c!='\n' && c!='\r' && c!=' ' && c!='\t' )
		{
			for (i=0;i<winSize-1;i++) undefPoint[i]=undefPoint[i+1];
			undefPoint[winSize-1]++;
		}
		
		/******/
	}
	if (c=='>') printf("Program treates only first sequence in fasta format");
	while (winPoint-startPoint /*<=*/< len_str-winSize )
	{
		extendOptEnergy(str+(winPoint-startPoint), winSize);

		extendZscore(n_smooth);
		calculateMinScore(0,Zthre);
		if (winPoint>=smooth_param) printLocOpt(out,n_locOpt,Zthre,0);
		if (print_minScore==1)printMinScore(minscoreFile,0);
		if (print_map==1)printMap(mapFile,0);
		winPoint++;
	}

	make_smooth(n_smooth,2);
	calculateMinScore(2,Zthre);
	printLocOpt(out,n_locOpt,Zthre,1);
	if (print_minScore==1)printMinScore(minscoreFile,1);
	if (print_map==1)printMap(mapFile,2);
	if (print_current==1 && winPoint%1000==0) printf("step\t%d\n",winPoint);
				
	if (vtimeFilename != NULL)
	{
		FILE* vtimeFile;
		vtimeFile=fopen(vtimeFilename,"w");
		if ( vtimeFile != NULL)
		{
			seq_process=(double)winPoint/f_length*100;
			if (timeNum==1) seq_process=seq_process/2;
			else if (timeNum==2) seq_process=50+seq_process/2;
			end_t=clock();
			time=(end_t-start_t)/CLOCKS_PER_SEC;
			time_left=(100-seq_process)/seq_process*time;
			fprintf(vtimeFile,"sequence process: %d%%\n",(int)seq_process);
			fprintf(vtimeFile,"time left: %d sec\n",(int)time_left);
		}
		fclose(vtimeFile);
	}
			
	free_param();
}



/* return sequence length until next delimiter ('>' or end of file) */
int len(FILE* f)
{
	int i=0;
	char c;
	while ( feof(f)==0 && (c=getc(f))!='>' )
	{
		if ( nuclNum(c)!=-1 ) i++;
		else if ( c!='\n' && c!='\r' && c!=' ' && c!='\t' && c!=-1 ) i++;
	}
	return i;
}

float E(int i,int j)
{
	return W[(i+winPoint)%winSize][(j+winPoint)%winSize];
}

float Z(int i,int j)
{
	return Zscore[(winSize+(i+winPoint)%winSize)%winSize][(winSize+(j+winPoint)%winSize)%winSize];
}

float ZbigScore(int i,int j)
{
	return Zbig[(i+winPoint)%(winSize+2*smooth_param)][(j+winPoint)%(winSize+2*smooth_param)];
}




/* initialize arrays */
void initializeParam(int n_smooth,int f_length)
{
	int i,k,delta=indent(winMin);

	/*** initialize matrix for energy calculation ***/
	V = (float**)malloc((winSize+1)*sizeof(float*));
	W = (float**)malloc((winSize+1)*sizeof(float*));
	VBI = (float**)malloc((winSize+1)*sizeof(float*));
	VM = (float**)malloc((winSize+1)*sizeof(float*));
	Zscore = (float**)malloc((winSize+1)*sizeof(float*));	
	Wcandidates = (int**)malloc((winSize+1)*sizeof(int*));
	VMcandidates = (float**)malloc((winSize+1)*sizeof(float*));
	lastWcand = (int*)malloc((winSize+1)*sizeof(int));

	for (k=0; k< winSize+1 ;k++) 
	{
		V[k] = (float*)malloc((winSize+1)*sizeof(float));
		W[k] = (float*)malloc((winSize+1)*sizeof(float));
		VBI[k] = (float*)malloc((winSize+1)*sizeof(float));
		VM[k] = (float*)malloc((winSize+1)*sizeof(float));
		Zscore[k] = (float*)malloc((winSize+1)*sizeof(float));
		
		Wcandidates[k] = (int*)malloc((winSize+1)*sizeof(int));
		VMcandidates[k] = (float*)malloc((winSize+1)*sizeof(float));
	}

	s.base = mallocate(winSize*sizeof(base));
	/*********/
	
	if (winSize>=d) 
	{
		if (d<=winMin) {len_par=winSize-winMin+1;len_str=winSize;}
		else {len_par=winSize-d+delta+1;len_str=winSize+delta;}
	}
	else {len_par=delta-indent(winSize)+1;len_str=delta+d-indent(winSize);}
	str = (char*)malloc(len_str*sizeof(char));
	if (len_str>f_length) len_str=f_length;
	//if (len_par>f_length) len_par=f_length;
	
	di = (int**)malloc(len_par*sizeof(int*));
	nucl = (int**)malloc(len_par*sizeof(int*));
	control_mean = (double**)malloc(len_par*sizeof(double*));
	control_sd = (double**)malloc(len_par*sizeof(double*));
	regressionNum = (int*)malloc(len_par*sizeof(int));
	startWin=(int*)malloc(len_par*sizeof(int));
	endWin=(int*)malloc(len_par*sizeof(int));
	undefPoint=(int*)malloc(winSize*sizeof(int));
	minScore=(float*)malloc(winSize*sizeof(float));
	factors=(float*)malloc((n_smooth+1)*sizeof(float));
	qp=(long double*)malloc((n_smooth+1)*sizeof(long double));
	for (i=0;i<len_par;i++)
	{
		di[i]=(int*)malloc(16*sizeof(int));
		nucl[i] = (int*)malloc(4*sizeof(int));
		control_mean[i] = (double*)malloc(control_points*sizeof(double));
		control_sd[i] = (double*)malloc(control_points*sizeof(double));
	}
	
	
	Zbig = (float**)malloc((winSize+2*n_smooth)*sizeof(float*));	
	for (k=0; k< winSize+2*n_smooth ;k++) 
	{
		Zbig[k] = (float*)malloc((winSize+2*n_smooth)*sizeof(float));	
	}
	
	Zsmooth = (float**)malloc((winSize+2*n_smooth)*sizeof(float*));	
	for (k=0; k< winSize+2*n_smooth ;k++) 
	{
		Zsmooth[k] = (float*)malloc((winSize+2*n_smooth)*sizeof(float));	
	}
	
	SL=(long double*)malloc((winSize-winMin+1)*sizeof(long double));
	SR=(long double*)malloc((winSize-winMin+1)*sizeof(long double));
	B1=(long double*)malloc((winSize-winMin+1)*sizeof(long double));
	B2=(long double*)malloc((winSize-winMin+1)*sizeof(long double));
	B3=(long double*)malloc((winSize-winMin+1)*sizeof(long double));
	B4=(long double*)malloc((winSize-winMin+1)*sizeof(long double));
	
	defLam(n_smooth);
	defStatWeight(n_smooth);
}

void initializeSurface(int n_smooth,double Zthre)
{
	int l,i,j,k,r,start,end,delta,n1,n2,n;
	double slope;
	delta=indent(winMin);
	for (r=0;r<len_par;r++)
	{	
		if (winSize<=d)
		{
			start=winSize+delta-r-d;
		}
		else if (winSize>d && winMin<=d)
		{
			if (r<=delta)
			{
				start=winSize+delta-r-d;
			}
			else
			{
				start=winSize-d-(r-delta);
			}
		}
		else
		{
			start=winSize-winMin-r;
			end=winSize-1;
		}
		
		if (start<0 || len_inpseq <=d ) start=0;
		if (start+d>=len_inpseq) start=len_inpseq-d;
		end=start+d-1;
		
		startWin[r]=start;
		endWin[r]=end;
		for (i=0;i<4;i++) nucl[r][i]=0;
		for (i=0;i<16;i++) di[r][i]=0;
		for (i=start;i<end;i++)
		{
			n1=nuclNum(str[i]);
			n2=nuclNum(str[i+1]);
			if ( n1!=-1 && n2!=-1 ) di[r][4*n1+n2]++;
			if (n1!=-1) nucl[r][n1]++;
		}
		if (nuclNum(str[end])!=-1) nucl[r][nuclNum(str[end])]++;
		
		regressionNum[r]=regressionNumber(nucl[r][0],nucl[r][1],nucl[r][2],nucl[r][3]);
		if (regressionNum[r]!=-1)
		{
			n=regressionNum[r];
			for (i=lower_ind;i<upper_ind+1;i++)
			{
				control_mean[r][i]=coef[i].mean[n][0];
				control_sd[r][i]=coef[i].sd[n][0];
				for (j=0;j<16;j++)
				{
					for (k=0;k<16;k++)
					{
						control_mean[r][i]+=coef[i].mean[n][j*16+k+1]*di[r][j]*di[r][k]/( (end-start)*(end-start) );
						control_sd[r][i]+=coef[i].sd[n][j*16+k+1]*di[r][j]*di[r][k]/( (end-start)*(end-start) );
					}
				}
			}
		}
	}

	
	/* here calculate Zscore and Zsmooth*/
	double mu,sd;
	int len;

	for (i=0;i<winSize+1;i++)
	{
		for (j=0;j<winSize+1;j++)	Zscore[i][j]=100;
	}

	for (i=0;i<winSize;i++)
	{
		for (j=i+winMin-1;j<winSize;j++)
		{
			len=j-i+1;
			if (useParams==0)
			{
				mu=meanEst(len, control_mean[len_par-1] );
				sd=sdEst(len, control_sd[len_par-1] );
					
				if (regressionNum[len_par-1]!=-1)
				{
					slope = slope_E;
					if ( pprob_exist==1 && uprob_exist==1 ) slope = slope + slope_E_cov*cover(i,j);
					Zscore[i][j]=( (W[i][j]-intercept)/slope+mu)/sd; 
					//printf("init %d %d %lf %lf %lf\n",i,j,W[i][j],Zscore[i][j],cover(i,j));
				}
			}
			else
			{
				Zscore[i][j]=(W[i][j]-expParam[len-winMin])/sdParam[len-winMin];
			}
		}
	}
	
		
	zdigits=2;
	if (smooth_param<=8) qdigits=2;
	else qdigits=1;
	
	for (i=0;i<winSize+2*smooth_param;i++)
	{
		for (j=0;j<winSize+2*smooth_param;j++)
		{
			Zbig[i][j]=0;
			if ( (i>=2*smooth_param) && (j>=2*smooth_param) && (j>=i)) 
			{
				if (Zscore[i-2*smooth_param][j-2*smooth_param] > 99) Zbig[i][j]=0;
				else Zbig[i][j]=nfloat(Zscore[i-2*smooth_param][j-2*smooth_param],zdigits);
			}
		}
	}
	
	
	float q=exp(-lambda);
	nq=nfloat(exp(-lambda),qdigits);
	for(l=0;l<n_smooth+1;l++)
	{
		factors[l]=pow(q,l);
		qp[l]=pow(nq,l)*pow(pow(10,qdigits),smooth_param-l);
	}
	make_smooth(n_smooth,1);
	
	/* here calculate minScore */
	calculateMinScore(1,Zthre);
}

/* Description: print locally optimal structures
	n_LocOpt - locally optimal parameter
	Zthre - Z-score threshold for structures output
	option: 0 if print only one column, 1 - print all matrix
*/
void printLocOpt(FILE* out,int n_LocOpt,double Zthre,int option)
{
	int i,j,inew,jnew,d_i,d_j;
	int flag;
	int start,end;
	float val,energy;
	if ( option==0 )
	{
		i=/*2**/smooth_param;
		for (j=i+winMin+1;j<winSize+/*2**/smooth_param;j++)
		{	
			if ( (smooth(i,j)<=Zthre && /*Zthre<=0*/accessible==0) || (smooth(i,j)>Zthre && /*Zthre>0*/accessible==1) )
			{
				flag=1;
				for (d_i=-n_LocOpt;d_i<n_LocOpt+1;d_i++)
				{
					for (d_j=-n_LocOpt+abs(d_i);d_j<n_LocOpt-abs(d_i)+1;d_j++)
					{
						if ( !(d_i==0 && d_j==0) && (i+d_i>=0) && (j+d_j<winSize+2*smooth_param) && (j+d_j-i-d_i>=winMin) )
						{
							int flag1=1;
							if ( smooth(i,j) >= smooth(i+d_i,j+d_j) && /*Zthre<=0*/accessible==0 ) {flag=0;flag1=0;}
							else if ( smooth(i,j) <= smooth(i+d_i,j+d_j) && /*Zthre>0*/accessible==1 ) {flag=0;flag1=0;}
						}
					}
				}
				
				if (flag==1)
				{	
					start=winPoint+i+1+undefPoint[i]-2*smooth_param;
					end=winPoint+j+1+undefPoint[i]-2*smooth_param;
					val=smooth(i,j);
					energy=E(i-2*smooth_param,j-2*smooth_param);
					fwrite(&start,sizeof(int),1,out);
					fwrite(&end,sizeof(int),1,out);
					fwrite(&energy,sizeof(float),1,out);
					fwrite(&val,sizeof(float),1,out);
				}
				
			}			
		}
	
	}
	else if (option==1)
	{

		for (i=smooth_param;i<winSize+2*smooth_param;i++)
		{
			if (  winPoint+i+1+undefPoint[i]-2*smooth_param >= 1 )
			{
			for (j=i+winMin+1;j<winSize+/*2**/smooth_param;j++)
			{

				if ( j-i+1 <= winSize )
				{
				
					flag=1;
					if ( (smooth(i,j)<=Zthre && accessible==0 ) || (smooth(i,j)>Zthre && accessible==1) ){
						for (d_i=-n_LocOpt;d_i<n_LocOpt+1;d_i++)
						{	
							for (d_j=-n_LocOpt+abs(d_i);d_j<n_LocOpt-abs(d_i)+1;d_j++)
							{
								if ( !(d_i==0 && d_j==0) && (i+d_i>=0) && (j+d_j<winSize+2*smooth_param) && (j+d_j-i-d_i>=winMin) )
								{
									int flag1=1;
									if ( smooth(i,j) >= smooth(i+d_i,j+d_j) && accessible==0 ){ flag=0;flag1=0;}
									else if ( smooth(i,j) <= smooth(i+d_i,j+d_j) && accessible==1 ) {flag=0;flag1=0;}
									
								}
							}
						}	
						
						if (flag==1)
						{	
							start=winPoint+i+1+undefPoint[i]-2*smooth_param;
							end=winPoint+j+1+undefPoint[i]-2*smooth_param;
							val=smooth(i,j);
							energy=E(i-2*smooth_param,j-2*smooth_param);				
							fwrite(&start,sizeof(int),1,out);
							fwrite(&end,sizeof(int),1,out);
							fwrite(&energy,sizeof(float),1,out);
							fwrite(&val,sizeof(float),1,out);
						}
					}					
				}
			}
		}
		}
	}
} 

/* calcualte minimum Z-score for positions
	option : 1 - calculate for sequence start, 0 - calcualte for new nucleotide
*/

void calculateMinScore(int option, double Zthre)
{
	int len,alpha,i,pos;
	if (option==0)
	{
		minScore[(winPoint+winSize-1)%winSize]=100;
		for (len=winMin+1;len<winSize;len++)
		{
			if (len % 2 == 1) 
			{
				alpha=(len-1)/2;
				i=winSize-1-alpha-smooth_param;
				
				if ( minScore[(i+winPoint)%winSize]>smooth(i-alpha+2*smooth_param,i+alpha+2*smooth_param) && Zthre<=0 )
					minScore[(i+winPoint)%winSize]=smooth(i-alpha+2*smooth_param,i+alpha+2*smooth_param);
				else if ( minScore[(i+winPoint)%winSize]>-smooth(i-alpha+2*smooth_param,i+alpha+2*smooth_param) && Zthre>0 )
					minScore[(i+winPoint)%winSize]=-smooth(i-alpha+2*smooth_param,i+alpha+2*smooth_param);
				
			}
		}
	}
	else if (option==1)
	{
		for (i=0;i<winSize;i++) minScore[i]=100;
		for (len=winMin;len<winSize+1;len++)
		{
			if (len % 2 == 1) 
			{
				alpha=(len-1)/2;
				for (pos=0;pos<winSize;pos++)
				{
					if ( (pos-alpha>=0) && (pos+alpha<winSize) )
					{
						if ( Zsmooth[pos-alpha+2*smooth_param][pos+alpha+2*smooth_param] < minScore[pos] && Zthre<=0 )
							minScore[pos]=Zsmooth[pos-alpha+2*smooth_param][pos+alpha+2*smooth_param];
						else if ( -Zsmooth[pos-alpha+2*smooth_param][pos+alpha+2*smooth_param] < minScore[pos] && Zthre>0 )
							minScore[pos]=-Zsmooth[pos-alpha+2*smooth_param][pos+alpha+2*smooth_param];
					}
				}
			}
		}
	}
	else if (option==2)
	{
		for (len=winMin;len<winSize+1;len++)
		{
			if (len % 2 == 1) 
			{
				alpha=(len-1)/2;
				for (i=winSize-1-alpha-smooth_param;i<winSize;i++)
				{	
					if ( (i-alpha>=0) && (i+alpha<winSize) )
					{
					if ( minScore[(i+winPoint)%winSize]>smooth(i-alpha+2*smooth_param,i+alpha+2*smooth_param) && Zthre<=0)
						minScore[(i+winPoint)%winSize]=smooth(i-alpha+2*smooth_param,i+alpha+2*smooth_param);
					else if ( minScore[(i+winPoint)%winSize]>-smooth(i-alpha+2*smooth_param,i+alpha+2*smooth_param) && Zthre>0)
						minScore[(i+winPoint)%winSize]=-smooth(i-alpha+2*smooth_param,i+alpha+2*smooth_param);
					}
				}
			}
		}
	}
	else
	{
		fprintf(stderr,"Wrong 'option' value");
		exit(1);
	}
}

void printMinScore(FILE* out, int opt)
{
	int i,pos;
	float val;
	if (opt==0) 
	{
		if ( minScore[winPoint%winSize]==100 ) minScore[winPoint%winSize]=0; 
		pos=winPoint+1+undefPoint[0];
		val=minScore[winPoint%winSize];
		fwrite(&pos,sizeof(int),1,out);
		fwrite(&val,sizeof(float),1,out);
	}
	if (opt==1)
	{
		for (i=0;i<winSize-1;i++) 
		{
			if ( minScore[(winPoint+i)%winSize]==100 ) minScore[(winPoint+i)%winSize]=0;
			pos=winPoint+1+i+undefPoint[i];
			val=minScore[(winPoint+i)%winSize];
			fwrite(&pos,sizeof(int),1,out);
			fwrite(&val,sizeof(float),1,out);
		}
	}
}


/* Print Z-scores to binary file for subsequent heatmap drawing
	option 0 - print central position
	option 1 - print start of the sequence
	option 2 - print end of the sequence
*/
void printMap(FILE* out, int option)
{
	int i,pos,h,alphaL,alphaR,x;
	char c;
	if ( winSize % 2 == 0 ) pos=winSize/2-1-smooth_param; else pos=(winSize-1)/2-smooth_param;
	
	if (option==0)
	{
		for (h=winMin-1;h<winSize;h++)
		{
			if ( h % 2 == 0 ) {alphaL=h/2;alphaR=alphaL;} 
			else {alphaL=(h-1)/2;alphaR=(h+1)/2;}
			c=convert( smooth(pos-alphaL+2*smooth_param,pos+alphaR+2*smooth_param) );
			if  (pos-alphaL +winPoint <= 0 || h<winMin ) c=(char)255;
			fwrite(&c,sizeof(char),1,out);
		}
	}
	
	else if (option==1)
	{
		for (i=0;i<pos+1;i++)
		{
			/* calcualte x: height of non-zero Z-scores in position i */
			x=2*(i-1);
			if ( winSize % 2 == 1 && i!=pos ) x++;
			if (x<winMin-1) x=winMin-2;
			
			for (h=winMin-1;h<x+1;h++)
			{
				if ( h % 2 == 0 ) {alphaL=h/2;alphaR=alphaL;} 
				else {alphaL=(h-1)/2;alphaR=(h+1)/2;}
				c=convert( smooth(i-alphaL+2*smooth_param,i+alphaR+2*smooth_param) ) ;
				fwrite(&c,sizeof(char),1,out);
			}
			
			for (h=x+1;h<winSize;h++) 
			{
				c=(char)255;
				fwrite(&c,sizeof(char),1,out);
			}
		}
	}
	
	else if (option==2)
	{
		for (i=pos;i<winSize;i++)
		{
			x = (winSize-1)-2*(i-pos/*+smooth_param*/);
			if (x<winMin-1) x=winMin-2;
			
			for (h=winMin-1;h<x+1;h++)
			{
				if ( h % 2 == 0 ) {alphaL=h/2;alphaR=alphaL;} 
				else {alphaL=(h-1)/2;alphaR=(h+1)/2;}
				c=convert( smooth(i-alphaL+2*smooth_param,i+alphaR+2*smooth_param) );
				fwrite(&c,sizeof(char),1,out);
			}
			
			for (h=x+1;h<winSize;h++)
			{
				//fprintf(out,"%c",255);
				c=(char)255;
				fwrite(&c,sizeof(char),1,out);
			}
		}
	}
	
	else 
	{
		fprintf(stderr,"Wrong 'option' value\n");
		exit(1);
	}
}


/* convert z-score to binary representation over char type 

*/
char convert(double z)
{
	/* Z-score bounds */
	double bound=10;
	/* Step for z-score rounding*/
	double step=0.1;
	if ( z > bound ) return 255;
	else if (z<-bound) return 0;
	else  return (int)( (z+bound)/step );
}


/* Calculate Z-score for sequence ended at new nucleotide */
void extendZscore(int n_smooth)
{
	int startInd,endInd,smoothInd,i,j,lenn,r;
	double mu,sd,slope;
	startInd=winPoint%winSize;
	endInd=(winPoint-1)%winSize;
	
	if (d>winSize) j=indent(winMin)-indent(winSize)+1;
	else if (d>winMin) j=indent(winMin);
	else j=0;
	
	for (i=0;i<winSize-winMin+1;i++)
	{
		lenn=winSize-i;
		if (lenn>=d) 
		{
			if (d>=winMin)r=j+(lenn-d);else r=j+(lenn-winMin);
		}
		else r=(j+(indent(winMin)-indent(lenn)-winPoint+1)%j)%j;
		//printf("%d %d %d\n",winSize,d,lenn);
		mu=meanEst(lenn,control_mean[r]);
		sd=sdEst(lenn,control_sd[r]);
		/*
		int ii;
		if (i==1){
			for (ii=0;ii<6;ii++) printf("%lf ",control_sd[r][ii]);
			printf("\n");
		}
		*/
		if (useParams==0)
		{
			slope = slope_E;
			if ( pprob_exist==1 && uprob_exist==1 ) slope = slope + slope_E_cov*cover(i,winSize-1);
			if (regressionNum[r]!=-1) Zscore[(startInd+i)%winSize][endInd]=( (E(i,winSize-1)-intercept)/slope +mu)/sd; 
			else {Zscore[(startInd+i)%winSize][endInd]=100;	}
			//printf("%lf %lf %lf %lf\n",E(i,winSize-1),mu,sd,Zscore[(startInd+i)%winSize][endInd]);
			//printf("extend %d %d %lf %lf %lf\n",(winPoint+i),(winPoint+winSize-1),E(i,winSize-1),Zscore[(startInd+i)%winSize][endInd],cover(i,winSize-1));
		}
		else
		{
			Zscore[(startInd+i)%winSize][endInd]=(E(i,winSize-1)-expParam[lenn-winMin])/sdParam[lenn-winMin];
		}
		if (Zscore[(startInd+i)%winSize][endInd]>99) Zbig[(winPoint+i+2*n_smooth)%(winSize+2*n_smooth)][(winPoint-1)%(winSize+2*n_smooth)]=0;
		else Zbig[(winPoint+i+2*n_smooth)%(winSize+2*n_smooth)][(winPoint-1)%(winSize+2*n_smooth)]=nfloat(Zscore[(startInd+i)%winSize][endInd],zdigits);
	}
	make_smooth(n_smooth,0);
}



void recalculate_param(char c)
{
	int l,k,m,s,e,nu,i0,j0,i,j,ldi;
	int ind,ind_prev,delta,startLen,pos;
	//printf("recalculate\n");
	if ( d>winMin )
	{				
		if (winSize<d) delta=indent(winMin)-indent(winSize)+1; else delta=indent(winMin);
		ind=(delta-winPoint%delta)%delta;
		ind_prev=(1+ind)%delta;
		//printf("delta: %d, ind: %d, ind_prev: %d winPoint: %d\n",delta,ind,ind_prev,winPoint);
		if ((winPoint+winSize+indent(winSize)>=d) && (endWin[ind_prev]<startPoint+len_str-1))
		{
			startWin[ind]=startWin[ind_prev]+1;
			endWin[ind]=endWin[ind_prev]+1;

			s=startWin[ind]-startPoint;
			e=endWin[ind]-startPoint;
			if (s==0)nu=nuclNum(c); else nu=nuclNum(str[s-1]);
			i0=4*nu+nuclNum(str[s]);
			j0=4*nuclNum(str[e-1])+nuclNum(str[e]);
			for (i=0;i<4;i++)nucl[ind][i]=nucl[ind_prev][i];
			for (i=0;i<4;i++)
			{
				for (j=0;j<4;j++)
				{
					di[ind][i*4+j]=di[ind_prev][i*4+j];
				}
			}
			nucl[ind][nu]--;
			nucl[ind][nuclNum(str[e])]++;
			di[ind][i0]--;
			di[ind][j0]++;
			ldi=endWin[ind]-startWin[ind];
			k=regressionNumber(nucl[ind][0],nucl[ind][1],nucl[ind][2],nucl[ind][3]);
			if (i0!=j0 && k!=-1)
			{
				if (k==regressionNum[ind_prev])
				{
					//x1++;
					for (i=lower_ind;i<upper_ind+1;i++)
					{
						control_mean[ind][i]=control_mean[ind_prev][i];
						control_sd[ind][i]=control_sd[ind_prev][i];
					}
			
					for (i=lower_ind;i<upper_ind+1;i++)
					{
						control_mean[ind][i]-=coef[i].mean[k][1+16*i0+i0]*(1+2*di[ind][i0])/( ldi*ldi );
						control_mean[ind][i]+=coef[i].mean[k][1+16*j0+j0]*(2*di[ind][j0]-1)/( ldi*ldi );
						control_mean[ind][i]+=(coef[i].mean[k][1+16*j0+i0]+coef[i].mean[k][1+16*i0+j0])*(di[ind][i0]-di[ind][j0]+1)/( ldi*ldi );
						for (j=0;j<16;j++)
						{
							if (j!=i0 && j!=j0)control_mean[ind][i]+=( coef[i].mean[k][1+16*j+j0] + coef[i].mean[k][1+16*j0+j] - coef[i].mean[k][1+16*i0+j] - coef[i].mean[k][1+16*j+i0] )*di[ind][j]/( ldi*ldi );
						}
				
						control_sd[ind][i]-=coef[i].sd[k][1+16*i0+i0]*(1+2*di[ind][i0])/( ldi*ldi );
						control_sd[ind][i]+=coef[i].sd[k][1+16*j0+j0]*(2*di[ind][j0]-1)/( ldi*ldi );
						control_sd[ind][i]+=(coef[i].sd[k][1+16*j0+i0]+coef[i].sd[k][1+16*i0+j0])*(di[ind][i0]-di[ind][j0]+1)/( ldi*ldi );
						for (j=0;j<16;j++)
						{
							if (j!=i0 && j!=j0)control_sd[ind][i]+=( coef[i].sd[k][1+16*j+j0] + coef[i].sd[k][1+16*j0+j] - coef[i].sd[k][1+16*i0+j] - coef[i].sd[k][1+16*j+i0] )*di[ind][j]/( ldi*ldi );
						}					
					}
				}
				else
				{
					for (i=lower_ind;i<upper_ind+1;i++)
					{
						control_mean[ind][i]=coef[i].mean[k][0];
						control_sd[ind][i]=coef[i].sd[k][0];
						for (j=0;j<16;j++)
						{
							for (m=0;m<16;m++)
							{
								control_mean[ind][i]+=coef[i].mean[k][j*16+m+1]*di[ind][j]*di[ind][m]/( ldi*ldi );
								control_sd[ind][i]+=coef[i].sd[k][j*16+m+1]*di[ind][j]*di[ind][m]/( ldi*ldi );
							}
						}
					}
				}
			}
			else
			{
				for (i=lower_ind;i<upper_ind+1;i++)
				{
					control_mean[ind][i]=control_mean[ind_prev][i];
					control_sd[ind][i]=control_sd[ind_prev][i];
				}
			}
			regressionNum[ind]=k;
		}
		else if ( !(endWin[ind_prev]<startPoint+len_str-2) )
		{
			startWin[ind]=startWin[ind_prev];
			endWin[ind]=endWin[ind_prev];
			for (i=0;i<4;i++)nucl[ind][i]=nucl[ind_prev][i];
			for (i=0;i<4;i++)
			{
				for (j=0;j<4;j++)
				{
					di[ind][i*4+j]=di[ind_prev][i*4+j];
				}
			}
			regressionNum[ind]=regressionNumber(nucl[ind][0],nucl[ind][1],nucl[ind][2],nucl[ind][3]);
			for (i=lower_ind;i<upper_ind+1;i++)
			{
				control_mean[ind][i]=control_mean[ind_prev][i];
				control_sd[ind][i]=control_sd[ind_prev][i];
			}
		}
	
	}
	
			
	if (winMin>=d) startLen=winMin; else startLen=d;
	if (winSize<d) delta=indent(winMin)-indent(winSize)+1; else delta=indent(winMin);
	if (d<=winMin) delta=0;
	for (l=startLen;l<winSize+1;l++)
	{
		pos=delta+l-startLen;
		
		/* condition, that can move dinucleotide window for fixed length l*/
		if ( (winPoint+winSize+indent(l)>=d || l>=d) && (endWin[pos]<startPoint+len_str-1) )
		{
			
			startWin[pos]++;
			endWin[pos]++;
			s=startWin[pos]-startPoint;
			e=endWin[pos]-startPoint;


			if (s==0)nu=nuclNum(c); else nu=nuclNum(str[s-1]);
			i0=4*nu+nuclNum(str[s]);
			j0=4*nuclNum(str[e-1])+nuclNum(str[e]);
		
			nucl[pos][nu]--;
			nucl[pos][nuclNum(str[e])]++;
			di[pos][i0]--;
			di[pos][j0]++;
			ldi=endWin[pos]-startWin[pos];
			k=regressionNumber(nucl[pos][0],nucl[pos][1],nucl[pos][2],nucl[pos][3]);

			if (i0!=j0 && k!=-1)
			{
				if (k==regressionNum[pos])
				{
					for (i=lower_ind;i<upper_ind+1;i++)
					{			
						control_mean[pos][i]-=coef[i].mean[k][1+16*i0+i0]*(1+2*di[pos][i0])/( ldi*ldi );
						control_mean[pos][i]+=coef[i].mean[k][1+16*j0+j0]*(2*di[pos][j0]-1)/( ldi*ldi );
						control_mean[pos][i]+=(coef[i].mean[k][1+16*j0+i0]+coef[i].mean[k][1+16*i0+j0])*(di[pos][i0]-di[pos][j0]+1)/( ldi*ldi );
						for (j=0;j<16;j++)
						{
							if (j!=i0 && j!=j0)control_mean[pos][i]+=( coef[i].mean[k][1+16*j+j0] + coef[i].mean[k][1+16*j0+j] - coef[i].mean[k][1+16*i0+j] - coef[i].mean[k][1+16*j+i0] )*di[pos][j]/( ldi*ldi );
						}
					
						control_sd[pos][i]-=coef[i].sd[k][1+16*i0+i0]*(1+2*di[pos][i0])/( ldi*ldi );
						control_sd[pos][i]+=coef[i].sd[k][1+16*j0+j0]*(2*di[pos][j0]-1)/( ldi*ldi );
						control_sd[pos][i]+=(coef[i].sd[k][1+16*j0+i0]+coef[i].sd[k][1+16*i0+j0])*(di[pos][i0]-di[pos][j0]+1)/( ldi*ldi );
						for (j=0;j<16;j++)
						{
							if (j!=i0 && j!=j0)control_sd[pos][i]+=( coef[i].sd[k][1+16*j+j0] + coef[i].sd[k][1+16*j0+j] - coef[i].sd[k][1+16*i0+j] - coef[i].sd[k][1+16*j+i0] )*di[pos][j]/( ldi*ldi );
						}			
					}
				}
				else
				{
					for (i=lower_ind;i<upper_ind+1;i++)
					{
						control_mean[pos][i]=coef[i].mean[k][0];
						control_sd[pos][i]=coef[i].sd[k][0];
						for (j=0;j<16;j++)
						{
							for (m=0;m<16;m++)
							{
								control_mean[pos][i]+=coef[i].mean[k][j*16+m+1]*di[pos][j]*di[pos][m]/( ldi*ldi );
								control_sd[pos][i]+=coef[i].sd[k][j*16+m+1]*di[pos][j]*di[pos][m]/( ldi*ldi );
							}
						}
					}
				}
			}
			regressionNum[pos]=k;
		}
	}
}


/* return fraction of covered bases of subsequence [i..j]*/
double cover(int i, int j){
	int k,pos;
	double cov;
	cov=0;
	for (k=i;k<j+1;k++)
	{
		pos=k+winPoint;
		if (uprob[pos] != -1 && pprob[pos] != -1) cov += 1 ;
	}
	cov=cov/(j-i+1);
	//printf("%lf\n",cov);
	return cov;
}


/* calculate indent to the left for determination dinucleotide composition for sequence of length l*/
int indent(int l)
{
	if (d<=l) return 0;
	else{
		if ( (d-l) % 2 == 0) return (d-l)/2;
		else return (d-l-1)/2;
	}
}

int nuclNum(char c1)
{
	if (c1=='a'||c1=='A') return 0;
	else if (c1=='t'||c1=='T'||c1=='u'||c1=='U') return 1;
	else if (c1=='g'||c1=='G') return 2;
	else if (c1=='c'||c1=='C') return 3;
	//else if (c1=='n' || c1=='N') return 4;
	else return -1;
}

int regressionNumber(int na,int nt,int ng,int nc)
{
	int gc,g_gc,a_at;
	if ( (ng+nc)==0 || (na+nt)==0 ) return -1;
	else{
		gc=5*(ng+nc)/(na+nt+ng+nc)-1;
		g_gc=5*ng/(ng+nc)-1;
		a_at=5*na/(na+nt)-1;
		if ( (gc>=0 && gc<=2) && (g_gc>=0 && g_gc<=2) && (a_at>=0 && a_at<=2) ) return (9*round(gc)+3*round(g_gc)+round(a_at));
		else return -1;
	}
	
}

void free_param()
{
	int i,k;
	free(str);
	free(regressionNum);
	free (startWin);
	free(endWin);
	free(undefPoint);
	free(minScore);
	for (i=0;i<len_par;i++)
	{
		free(di[i]);
		free(nucl[i]);
		free(control_mean[i]);
		free(control_sd[i]);
	}
	free(di);
	free(nucl);
	free(control_mean);
	free(control_sd);

	for (k=0; k<(winSize+1) ;k++) {
		
		free(V[k]);
		free(W[k]);
		free(VBI[k]);
		free(VM[k]);

		free(Wcandidates[k]);
		free(VMcandidates[k]);
		
		free(Zscore[k]);
		//free(Zsmooth[k]);
	}

	for (k=0;k<winSize+2*smooth_param;k++) free(Zsmooth[k]);

	free(V);
	free(W);
	free(VBI);
	free(VM);
	free(Zscore);
	free(Zsmooth);

	free(Wcandidates);
	free(VMcandidates);
	free(lastWcand);	
	free(s.base);
	
	free(factors);
	free(qp);
	free(SL);
	free(SR);
	free(B1);
	free(B2);
	free(B3);
	free(B4);
	
	free(statWeight);
	for (k=0; k< winSize+2*smooth_param ;k++) free(Zbig[k]);
	free(Zbig);
	
}

int main(int argc, char* argv[])
{	
	char* inFilename;
	char* outFilename;
	char* mapFilename;
	char* signFilename;
	char* densFilename;
	char* vtimeFilename;
	char *paraminFilename=NULL;
	char *paramregFilename=NULL;
	char mapBinary[255],mapKeys[255];
	char *pprobFilename=NULL, *uprobFilename=NULL, *reactFilename=NULL;
	/*** string for stdin ***/
	char seqin[255];
	char name[255];
	/******/
	double Zthre;
	int winSize_store,winMin_store;
	int n_smooth;
	int n_locOpt;
	int length;
	int vstep;
	int option;
	int densFrame;
	int print_current=0,print_map=0,print_sign=0,print_dens=0,print_vtime=0;	
	/*** param for probing data ***/
	struct energy_sample* smpl;
	int sample_size=200;
	double pparam=0.01;
	/******************************/
	
	/*
		timeNum =
			0 make forward strand, only forward strand exists
			1 make forward strand, reverse strand exists
			2 make reverse strand
	*/
	int timeNum=0;
	FILE *inp, *outp, *mapFile,*mapFileKeys, *signFile,*densFile;
	int i;
	struct InputParam inPar;
	char path[PATH_LEN];
	start_t=clock();
	/*** read config file and initialize variables ***/	
	
	if ( strcmp(PLATFORM,"win") ==0 )
	{
		GetModuleFileName( NULL, path, PATH_LEN + 1 );
	}
	else
	{
		realpath (argv[0], path);
	}
	
	parseConfig(path,&inPar);
	winSize=inPar.winMax;
	winMin=inPar.winMin;
	d=inPar.diWindow;
	inFilename=inPar.inFile;
	outFilename=inPar.outFile;
	Zthre=inPar.z;
	n_smooth=inPar.smooth;
	n_locOpt=inPar.locOpt;
	print_current=inPar.v;
	vstep=inPar.vstep;
	print_dens=inPar.print_dens;
	print_map=inPar.print_map;
	print_sign=inPar.print_sign;
	signFilename=inPar.signFile;
	mapFilename=inPar.mapFile;
	densFilename=inPar.densFile;
	densFrame=inPar.densFrame;
	chrName=inPar.chr;
	char* head=(char*)malloc(PATH_LEN*sizeof(char));
	pathHead(path,head);
	/*read_coefficients(head);
	read_tables(head);
	free(head);*/
	accessible=0;
	/******/
	
	/*** read parameters from command line trwe***/
	struct gengetopt_args_info args_info;
	
	if (cmdline_parser(argc, argv, &args_info) != 0)
    {
		fprintf(stderr,"Wrong command line options\n");
		exit(1);
	}
	if ( args_info.input_given ) inFilename=args_info.input_arg;
	if ( args_info.output_given ) outFilename=args_info.output_arg;
	if ( args_info.diframe_given ) d=args_info.diframe_arg;	
	if ( args_info.winmax_given ) winSize=args_info.winmax_arg;	
	if ( args_info.winmin_given ) winMin=args_info.winmin_arg;	
	if ( args_info.z_given ) Zthre=args_info.z_arg;	
	if ( args_info.structured_given ) accessible=args_info.structured_flag;
	if ( args_info.accessible_given ) accessible=args_info.accessible_flag;
	if ( args_info.smooth_given ) n_smooth=args_info.smooth_arg;	
	if ( args_info.locopt_given ) n_locOpt=args_info.locopt_arg;	
	if ( args_info.mapfile_given ) 
	{
		mapFilename=args_info.mapfile_arg;	
		print_map=1;
	}
	if ( args_info.signfile_given ) 
	{
		signFilename=args_info.signfile_arg;	
		print_sign=1;
	}
	if ( args_info.densfile_given ) 
	{
		densFilename=args_info.densfile_arg;	
		print_dens=1;
	}
	if ( args_info.densframe_given ) densFrame=args_info.densframe_arg;
	if ( args_info.v_given ) print_current=args_info.v_flag;
	if (args_info.vtimefile_given) 
	{
		vtimeFilename=args_info.vtimefile_arg;
		print_vtime=1;
	}
	if (args_info.vstep_given) vstep=args_info.vstep_arg;
	if ( args_info.chr_given ) chrName=args_info.chr_arg;
	
	/* experimental data */
	if ( args_info.paired_given ) pprobFilename=args_info.paired_arg;
	if ( args_info.unpaired_given ) uprobFilename=args_info.unpaired_arg;
	if ( args_info.reactivity_given ) reactFilename=args_info.reactivity_arg;
	if ( args_info.weight_given ) exp_const=args_info.weight_arg;
	
	/* param data */
	if ( args_info.paramin_given ) 	paraminFilename=args_info.paramin_arg;
	if ( args_info.paramreg_given ) 	paramregFilename=args_info.paramreg_arg;
	if ( args_info.sample_size_given ) sample_size=args_info.sample_size_arg;
	if ( args_info.reg_param_given ) pparam=args_info.reg_param_arg;

	/******/

	/*** check file and path correctness ***/
	
	if ( (inp = fopen(inFilename, "r" )) == NULL )
	{
		fprintf(stderr,"Error in opening input file for read\n");
		exit( 2 );
	}

	if ( (outp = fopen(outFilename, "w" )) == NULL )
	{
		printf("Error in opening output file for write\n");
		exit( 3 );
	}

	if (print_map==1) 
	{
		sprintf(mapBinary,"%s_binary",mapFilename);
		sprintf(mapKeys,"%s_keys",mapFilename);
		if ( (mapFile = fopen(mapBinary,"w+b")) ==NULL )
		{
			fprintf(stderr,"Error in opening map binary file for write\n");
			exit( 4 );
		}
		if ( (mapFileKeys = fopen(mapKeys,"w")) ==NULL )
		{
			fprintf(stderr,"Error in opening map keys file for write\n");
			exit( 5 );
		}
		/* print two parameters for heatmap pictures: winSize-winMin+1 - width of rectangle, winMin-1 - distance to sequnce line */
		//fprintf(mapFileKeys,"%d\n%d",(winSize-winMin+1),(winMin-1));
	}else mapFile=NULL;
	
	if (print_sign==1) 
	{
		if ( (signFile = fopen(signFilename,"w")) ==NULL )
		{
			fprintf(stderr,"Error in opening score file for write\n");
			exit( 6 );
		}
		
	}else signFile=NULL;
	
	if (print_dens==1) 
	{
		if ( (densFile = fopen(densFilename,"w")) ==NULL )
		{
			fprintf(stderr,"Error in opening density file for write\n");
			exit( 7 );
		}
		
	}else densFile=NULL;
	
	if (print_vtime==1)
	{
		if ( fopen(vtimeFilename,"w")==NULL )
		{
			fprintf(stderr,"Error in opening time file");
			exit(1);
		}
	}
	/***/
	
	if (winMin<length_low_bound)
	{
		fprintf(stderr,"lower window length must be greater than %d\n",length_low_bound);
		exit(8);
	}
	if (winMin>winSize)
	{
		fprintf(stderr,"upper window length %d must be greater than lower %d",winSize,winMin);
		exit(9);
	}
	if (d<0)
	{
		fprintf(stderr,"Parameter d must be greater than 0\n");
		exit(10);
	}
	if (densFrame<winMin)
	{
		fprintf(stderr,"Parameter densFrame must be greater then winMin\n");
		exit(11);
	}
	
	if (n_smooth>16)  n_smooth=16;
	smooth_param=n_smooth;

	/* define binary files */
	
	char loBinFilename[PATH_LEN],signBinFilename[PATH_LEN],densBinFilename[PATH_LEN];
	FILE *loBinFile,*signBinFile,*densBinFile;
	
	sprintf(loBinFilename,"%s+",outFilename);
	sprintf(signBinFilename,"%s+",signFilename);
	sprintf(densBinFilename,"%s+",densFilename);
	
	if ( (loBinFile = fopen(loBinFilename,"w+b")) ==NULL )
	{
		fprintf(stderr,"Error in opening binary output file for write\n");
		exit( 12 );
	}
	
	if (print_sign==1) 
	{
		if ( (signBinFile = fopen(signBinFilename,"w+b")) ==NULL )
		{
			fprintf(stderr,"Error in opening binary sign file for write\n");
			exit( 13 );
		}	
	}else signBinFile=NULL;
	
	char* inFilenameC=(char*)malloc( ( strlen(inFilename)+2 )*sizeof(char) );
	FILE* inpC;
	sprintf(inFilenameC,"%sc",inFilename);
	
	if ( fopen(inFilenameC,"r") !=NULL ) timeNum++; 
	
	
	/******************* Estimate slope and intercept of energies with probing data ***********/
	winSize_store=winSize;
	winMin_store=winMin;
	if ( !args_info.unmap_given & args_info.paired_given & args_info.unpaired_given ){
		if (  args_info.paramreg_given  ){
			FILE* inputreg_file;
			if ( (inputreg_file = fopen(paramregFilename,"r")) == NULL ){
				fprintf(stderr,"Error in opening file with regression coefficients for write\n");
				exit( 30 );
			}
			fscanf(inputreg_file,"%lf %lf %lf",&slope_E_cov,&slope_E,&intercept);
			fclose(inputreg_file);
		}
		else{
			smpl=(struct energy_sample*)malloc( sample_size*sizeof(struct energy_sample));
			sample_energies_general( inFilename, pprobFilename, uprobFilename, winMin, winSize, sample_size, pparam, smpl, argv[0]);
			estimateRegPar(smpl,sample_size,&slope_E,&intercept);
			free(smpl);
			slope_E_cov=0;
		}
	}
	else{ 
		slope_E_cov=0;
		slope_E=1;
		intercept=0;
	}
	if ( args_info.out_reg_given ){	
		FILE* reg_file;
		if ( (reg_file = fopen(args_info.out_reg_arg,"w")) ==NULL )
		{
			fprintf(stderr,"Error in opening reg_file file for write\n");
			exit( 30 );
		}
		fprintf(reg_file,"slope_E_cov %lf\nslope_E %lf\nintercept %lf\n",slope_E_cov,slope_E,intercept);	
		fclose(reg_file);
	}

	
	/*****************************************************************************************/
	winSize=winSize_store;
	winMin=winMin_store;

	read_coefficients(head);
	read_tables(head);
	free(head);
	/***** Initialization of arrays with experimantal data *****/		
	length=seqFileLength(inFilename);
	uprob_exist=0;pprob_exist=0;react_exist=0;
	if ( uprobFilename!=NULL ){
		initialize_exp(uprobFilename,'u',length);
	}
	if ( pprobFilename!=NULL ){
		initialize_exp(pprobFilename,'p',length);	

	}
	else if ( uprobFilename==NULL && pprobFilename==NULL && reactFilename!=NULL )
		initialize_exp(reactFilename,'r',length);		
	/******************************************************************/	
	
	if ( args_info.paramin_given )
		initializeEParam(2,paraminFilename,&useParams);
	else
		initializeEParam(0,NULL,&useParams);
	zSurface(inp,loBinFile,Zthre,n_smooth,n_locOpt,print_current,vtimeFilename,vstep,timeNum,print_map,mapFile,mapFileKeys,print_sign,signBinFile);
	if (print_dens==1) densityLocOpt(loBinFile,densBinFilename,seqFileLength(inFilename),densFrame);	
	
	if (print_map==1) fclose(mapFile);
	if (print_sign==1) fclose(signBinFile);
	fclose(loBinFile);

		/*** here make "-" strand ***/ 
		

		if ( (inpC=fopen(inFilenameC,"r")) !=NULL )
		{
			char loBinFilenameC[PATH_LEN],signBinFilenameC[PATH_LEN],densBinFilenameC[PATH_LEN],mapBinaryC[PATH_LEN];
			FILE *loBinFileC,*signBinFileC,*mapFileC;
			sprintf(loBinFilenameC,"%s-",outFilename);
			sprintf(signBinFilenameC,"%s-",signFilename);
			sprintf(densBinFilenameC,"%s-",densFilename);

			if ( (loBinFileC = fopen(loBinFilenameC,"w+b")) ==NULL )
			{
				fprintf(stderr,"Error in opening binary output file for write\n");
				exit( 12 );
			}
		
			if (print_sign==1) 
			{
				if ( (signBinFileC = fopen(signBinFilenameC,"w+b")) ==NULL )
				{
					fprintf(stderr,"Error in opening binary sign file for write\n");
					exit( 13 );
				}	
			}else signBinFileC=NULL;
			
			if (print_map==1) 
			{
				sprintf(mapBinaryC,"%s_binary-",mapFilename);
				if ( (mapFileC = fopen(mapBinaryC,"w+b")) ==NULL )
				{
					fprintf(stderr,"Error in opening map binary file for write\n");
					exit( 14 );
				}	
			}else mapFile=NULL;
				
			timeNum++;
			zSurface(inpC,loBinFileC,Zthre,n_smooth,n_locOpt,print_current,vtimeFilename,vstep,timeNum,print_map,mapFileC,NULL,print_sign,signBinFileC);
			
			if (print_dens==1) densityLocOpt(loBinFileC,densBinFilenameC,seqFileLength(inFilenameC),densFrame);
			
			
			/*****************************************/
			/*
			FILE *signFileC=fopen("output/signC","w"),*densFileC=fopen("output/densC","w");	
			FILE* densBinFileC;
			if (print_dens==1) 
			{
				if ( (densBinFileC = fopen(densBinFilenameC,"rb")) ==NULL )
				{
					fprintf(stderr,"Error in openning binary density file for write\n");
					exit( 140 );
				}	
			}else densBinFileC=NULL;
			
			if (print_sign==1)
			{
				fileConverter(chrName,signBinFileC,signFileC,0);
				fclose(signFileC);
			}
			
			if (print_dens==1)
			{
				fileConverter(chrName,densBinFileC,densFileC,1);
				fclose(densFileC);
			}
			*/
			/*****************************************/
			
			
			if (print_map==1) 
			{
				fclose(mapFileC);
				mergeStrandsMap(mapBinary,mapBinaryC,seqFileLength(inFilenameC),winSize-winMin+1);
				remove(mapBinaryC);
			}
			if (print_sign==1)
			{
				fclose(signBinFileC);
				mergeStrands(signBinFilename,signBinFilenameC,seqFileLength(inFilenameC),0);
				remove(signBinFilenameC);
			}
		
			if (print_dens==1)
   			{
				mergeStrands(densBinFilename,densBinFilenameC,seqFileLength(inFilenameC),1);
				remove(densBinFilenameC);
			}
			fclose(loBinFileC);
			fclose(inpC);
			
			makeBedLocOpt(loBinFilename,loBinFilenameC,outp,chrName,seqFileLength(inFilenameC),1);
			remove(loBinFilename);
			remove(loBinFilenameC);
			
		}
		else
		{
			makeBedLocOpt(loBinFilename,NULL,outp,chrName,0,0);
			remove(loBinFilename);
		}
		/***					***/
	
	
	if (print_dens==1) 
	{
		if ( (densBinFile = fopen(densBinFilename,"rb")) ==NULL )
		{
			fprintf(stderr,"Error in opening binary density file for write\n");
			exit( 15 );
		}	
		
		fileConverter(chrName,densBinFile,densFile,1);
		fclose(densFile);
		fclose(densBinFile);
		remove(densBinFilename);
	}
	if (print_sign==1)
	{
		if ( (signBinFile = fopen(signBinFilename,"rb")) ==NULL )
		{
			fprintf(stderr,"Error in opening binary sign file for write\n");
			exit( 13 );
		}	
		fileConverter(chrName,signBinFile,signFile,0);
		fclose(signFile);
		fclose(signBinFile);
		remove(signBinFilename);
	}

	//if (print_map==1) fclose(mapFile);
	
	fclose(inp);fclose(outp);
	free(inFilenameC);
	free_tables();
	end_t=clock();
	//printf("\ntime %lf\n",(end_t-start_t)/CLOCKS_PER_SEC);

	return 0;
}


/***********************/
/***********************/
/*** make smooth ***/

/* return position (i,j) of smoothed Z-score */
float smooth(int i,int j)
{
	return Zsmooth[(i+winPoint)%(winSize+2*smooth_param)][(j+winPoint)%(winSize+2*smooth_param)];
}

/* calculate smooth Z-score for position (i,j) by simple bust */
float calculate_smooth(int i,int j,int n_smooth)
{
	float sum,factor,Zsm;
	int d_i,d_j,i_z,j_z;
	sum=0;
	Zsm=0;
	for (d_i=-n_smooth;d_i<n_smooth+1;d_i++)
	{
		for (d_j=-n_smooth+abs(d_i);d_j<n_smooth-abs(d_i)+1;d_j++)
		{
			i_z=i+d_i;
			j_z=j+d_j;
			if ( ( i_z >= 0 ) && ( j_z < winSize+2*n_smooth ) && ( j_z-i_z>=(winMin-1) ) )
			{
				if (n_smooth != 0) factor= factors[abs(d_i)+abs(d_j)]; else factor=1;
				Zsm+=factor*ZbigScore(i_z,j_z);
				sum+=factor;
			}
		}
		
	}
	if (sum==0) return 0; else return Zsm/sum/pow(10,zdigits);
}


/* Calculate smooth Z-score surface 
	n_smooth - smooth parameter
	option 0 - calculate one column, 1 - calculate all matrix
*/
void make_smooth(int n_smooth,int option)
{
	int i,j,posj,x;
	int startInd,endInd;
	int switchSmooth=10;
	
	
	if (option==1)
	{
		for (i=0;i<winSize+2*n_smooth;i++)
		{
			for (j=0;j<winSize+2*n_smooth;j++)
			{
				Zsmooth[i][j]=calculate_smooth(i,j,n_smooth);
			}
		}
		initArrays(winSize-1+n_smooth);
	}
	else if (option==2)
	{
		if (n_smooth<switchSmooth)
		{
			for (posj=winSize+n_smooth-1; posj<winSize+2*n_smooth-1; posj++){
				startInd=winPoint%(winSize+2*n_smooth);
				endInd=(winPoint+posj)%(winSize+2*n_smooth);
				for (i=posj-winSize+1;i<posj-winMin+1;i++)
				{
					Zsmooth[(startInd+i)%(winSize+2*n_smooth)][endInd]=calculate_smooth(i,posj,n_smooth);
				}
			}
		}
	}
	else if (option==0)
	{
		if (n_smooth<switchSmooth)
		{
			posj=winSize+n_smooth-1;
			startInd=winPoint%(winSize+2*n_smooth);
			endInd=(winPoint+posj)%(winSize+2*n_smooth);
			for (i=posj-winSize+1;i<posj-winMin+1;i++)
			{
				Zsmooth[(startInd+i)%(winSize+2*n_smooth)][endInd]=calculate_smooth(i,posj,n_smooth);
			}
		}
		else
		{
			
			step(winSize-1+n_smooth);
			double aa;
			for (x=winMin;x<winSize;x++)
			{
				Zsmooth[(winSize-1+n_smooth-x+winPoint)%(winSize+2*n_smooth)][(winSize-1+n_smooth+winPoint)%(winSize+2*n_smooth)]=(float)((SL[winSize-1-x]+SR[winSize-1-x])*pow(10,-smooth_param*qdigits)*pow(10,-zdigits))/statWeight[0];
			}
			
		}

	}
		
}

initArrays(int pos_j)
{
	int i,i1,j1,i2;
	
	for (i=pos_j-winSize+1;i<pos_j-winMin+1;i++)
	{
		
		/* calculate B1 */
		B1[i-(pos_j-winSize+1)]=0;
		if (pos_j-smooth_param-i+1>=winMin)
		{
			for (i1=i;i1<i+smooth_param+1;i1++)
			{
				B1[i-(pos_j-winSize+1)]+=ZbigScore(i1,pos_j-smooth_param+i1-i);
			}
			B1[i-(pos_j-winSize+1)]=B1[i-(pos_j-winSize+1)]*qp[smooth_param];
		}
		
		/* calculate B2 */
		B2[i-(pos_j-winSize+1)]=0;
		for (i1=i;i1>i-smooth_param-1;i1--)
		{
			B2[i-(pos_j-winSize+1)]+=ZbigScore(i1,pos_j-smooth_param+i-i1);
		}
		B2[i-(pos_j-winSize+1)]=B2[i-(pos_j-winSize+1)]*qp[smooth_param];
		
		/* calculate B3 */
		B3[i-(pos_j-winSize+1)]=0;
		for (i1=i;i1<i+smooth_param;i1++)
		{
			if (pos_j+smooth_param+i-2*i1+1>=winMin) B3[i-(pos_j-winSize+1)]+=ZbigScore(i1,pos_j+smooth_param+i-i1);
		}
		B3[i-(pos_j-winSize+1)]=B3[i-(pos_j-winSize+1)]*qp[smooth_param];
		
		/* calculate B4 */
		B4[i-(pos_j-winSize+1)]=0;
		if (pos_j+smooth_param-i+1<=winSize)
		{
			for (i1=i;i1>i-smooth_param;i1--)
			{
				B4[i-(pos_j-winSize+1)]+=ZbigScore(i1,pos_j+smooth_param+i1-i);
			}
			B4[i-(pos_j-winSize+1)]=B4[i-(pos_j-winSize+1)]*qp[smooth_param];
		}
		
		/* calculate SL */
		SL[i-(pos_j-winSize+1)]=0;
		for (i1=-smooth_param;i1<smooth_param+1;i1++)
		{
			for (j1=-smooth_param+abs(i1);j1<1;j1++)
			{
				if ( pos_j+j1-i-i1 >= winMin-1 && pos_j+j1-i-i1<winSize ) 
					SL[i-(pos_j-winSize+1)]+=ZbigScore(i+i1,pos_j+j1)*qp[abs(i1)+abs(j1)];
			}
		}
		/* calculate SR */
		SR[i-(pos_j-winSize+1)]=0;
		for (i1=-smooth_param;i1<smooth_param+1;i1++)
		{
			for (j1=1;j1<smooth_param-abs(i1)+1;j1++)
			{
				if ( pos_j+j1-i-i1 >= winMin-1 && pos_j+j1-i-i1<winSize ) 
					SR[i-(pos_j-winSize+1)]+=ZbigScore(i+i1,pos_j+j1)*qp[abs(i1)+abs(j1)];
			}
		}
	}
}

void step(int pos_j)
{
	int i,i1,i2,j2;
	
	/* calculate partly SL and B2,B3 arrays*/
	for (i=pos_j-winSize+1;i<pos_j-winMin-1;i++)
	{
		i1=i-(pos_j-winSize+1);
		
		/* partly calcualte SL */
		SL[i1]=(SL[i1+1] -B1[i1+1]-B2[i1+1] + ZbigScore(i,pos_j-smooth_param-1)*qp[smooth_param] ) * nq/pow(10,qdigits);
		
		/* calculate B2 and B3 arrays */
		B2[i1]=B2[i1+2]+( ZbigScore(i-smooth_param,pos_j)-ZbigScore(i+1,pos_j-smooth_param-1) )*qp[smooth_param];
		B3[i1]=B3[i1+2]+( ZbigScore(i,pos_j+smooth_param)-ZbigScore(i+smooth_param,pos_j) )*qp[smooth_param];
	}

	/* calcualtes B1 and B4*/
	for (i=pos_j-winMin;i>pos_j-winSize+1;i--)
	{
		i1=i-(pos_j-winSize+1);
		if (pos_j-smooth_param-i+1>=winMin) 
			B1[i1]=B1[i1]+( ZbigScore(i+smooth_param,pos_j)-ZbigScore(i-1,pos_j-smooth_param-1) )*qp[smooth_param];
		
		if (pos_j+smooth_param-i+1<=winSize) 
			B4[i1]=B4[i1]+( ZbigScore(i,pos_j+smooth_param)-ZbigScore(i-smooth_param,pos_j) )*qp[smooth_param];
	}
	
	/* calculate B2,B3 for i=pos_j-winMin;pos_j-winMin-1; */
	for(i=pos_j-winMin-1;i<pos_j-winMin+1;i++)
	{
		i1=i-(pos_j-winSize+1);
		B2[i1]=0;
		for (i2=i;i2>i-smooth_param-1;i2--)
		{	
			B2[i1]+=ZbigScore(i2,pos_j-smooth_param+i-i2);
		}
		B2[i1]=B2[i1]*qp[smooth_param];
		B3[i1]=0;
		for (i2=i;i2<i+smooth_param+1;i2++)
		{
			if (pos_j+smooth_param+i-2*i2+1>=winMin)
				B3[i1]+=ZbigScore(i2,pos_j+smooth_param+i-i2);
		}
		B3[i1]=B3[i1]*qp[smooth_param];
	}

	
	long double down,up;
	double down0,up0;
	for (i=pos_j-winSize+1;i<pos_j-winMin-1;i++)
	{
		/* calculates down and up */
		i1=i-(pos_j-winSize+1);
		if (i1==0)
		{
			down=ZbigScore(i,pos_j)*qp[0];
			up=0;
			for (i2=i+1;i2<i+smooth_param+1;i2++) up+=ZbigScore(i2,pos_j)*qp[i2-i];
		}
		else
		{
			if ( i-smooth_param-1>=pos_j-winSize+1 ) down=down-ZbigScore(i-smooth_param-1,pos_j)*qp[smooth_param];
			down=down*nq/pow(10,qdigits)+ZbigScore(i,pos_j)*qp[0];

			up=up*pow(10,qdigits)/nq;	
			up+=ZbigScore(i+smooth_param,pos_j)*qp[smooth_param]-ZbigScore(i,pos_j)*qp[0];	
		}
		
		/* calculates SL,SR*/

		SL[i1]+=up+down;
	
		SR[i1]=( SR[i1+1]+(-up-down+( ZbigScore(i-smooth_param,pos_j)+ZbigScore(i+smooth_param,pos_j) )*qp[smooth_param])*nq/pow(10,qdigits) );
		SR[i1]=SR[i1]*pow(10,qdigits)/nq;
		SR[i1]+=B3[i1]+B4[i1]-ZbigScore(i,pos_j+smooth_param)*qp[smooth_param];
	}

	
	/* calculate SL and SR for i=pos_j-winMin-1;pos_j-winMin*/
	for(i=pos_j-winMin;i>pos_j-winMin-2;i--)
	{
		i1=i-(pos_j-winSize+1);
		SL[i1]=0;
		for (i2=-smooth_param;i2<smooth_param+1;i2++)
		{
			for (j2=-smooth_param+abs(i2);j2<1;j2++)
			{
				if ( pos_j+j2-i-i2 >= winMin-1 && pos_j+j2-i-i2<winSize ) 
					SL[i1]+=ZbigScore(i+i2,pos_j+j2)*qp[abs(i2)+abs(j2)];
			}
		}
		SR[i1]=0;
		for (i2=-smooth_param;i2<smooth_param+1;i2++)
		{
			for (j2=1;j2<smooth_param-abs(i2)+1;j2++)
			{
				if ( pos_j+j2-i-i2 >= winMin-1 && pos_j+j2-i-i2<winSize ) 
					SR[i1]+=ZbigScore(i+i2,pos_j+j2)*qp[abs(i2)+abs(j2)];
			}
		}
	}

	
}

long double nfloat(float e,int n)
{
	return round(e*pow(10,n));
}

long double qpow(int qq,int k)
{
	return ( pow(qq,k)*pow(100,smooth_param-k) );
}

int seqFileLength(char* filename)
{
		
	
	FILE* in;
	char c;
	int length;
	fpos_t f_pos;
	char name[256];


	if ( (in=fopen(filename,"r")) == NULL )
	{
		fprintf(stderr,"Cannot open file to read sequence length\n");
		exit(1);
	}
	
	fscanf(in,"%s",name);
	if (name[0]!='>')
	{
		fprintf(stderr,"Input sequence must be in fasta format");
		exit(16);
	}
	while ( (c=fgetc(in))!='\n' && c!='\r'  )
		;
		
	length=len(in);	
	fclose(in);	
	return length;
}