/* 
 * Authors: Ydo Wexler & Michal Ziv-Ukelson 2008
 * RNA secondary structure prediction of all consecutive sliding windows in a sequence.
 * The algorithm is based on the papers "A Study of Accessible Motifs and RNA Folding Complexity" 
 by Ydo Wexler, Chaya Zilberstein and Michal Ziv-Ukelson (JCB, 2007), 
 and the paper  "A fast algorithm to scan the RNA folding energy landscape of genomes 
 and its application to the analysis of genomic folding asymmetry" 
 by Yair Horesh, Ydo Wexler, Michal Ziv-Ukelson, and Ron Unger.

 Anyone using, modyfying, or redistributing this code is required to reference these works.
 Moreover, the copywrite notice must not be removed with any manipulation / redistribution of the code
 or the software.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE 
 FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
 ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
 EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *              
 */


#ifndef ENERGY

#define ENERGY
#define ENERGYSCANFORMAT "%f"
#ifdef INTEGER
#define ENERGYPRINTFORMAT "%i"
#else
#define ENERGYPRINTFORMAT "%.1f"
#endif

#include "../common.h"

energy *loop; /* Staility of interior loops indexed by size */
energy *bulge; /* Stability of bulges indexed by size */
energy *hairpin; /* Stability of hairpin loops indexed by size */
energy dangle3[MISMATCH + 1][UNDEF + 1]; /* Dangling (unpaired) base stacking
					  * effects at 3' end
					  */
energy dangle5[MISMATCH + 1][UNDEF + 1]; /* Dangling (unpaired) base stacking
					  * effect at the 5' end
					  */
energy *f; /* Array for function f in Ninio correction */
energy mloop_a; /* Multibranched loop closing constant */
energy mloop_b; /* Multibranched loop stem constant */
energy mloop_c; /* Multibranched loop unpaired base constant */
energy ninio_max_correction; /* Maximum asymmetry correction by Ninio rules */
float logparam; /* Parameter for logarithmic extrapolation for loops and
		 * bulges; always represented by a float to postpone rounding.
		 */
energy stack[MISMATCH + 1][MISMATCH + 1]; /* Base pair stacking table */
energy tetraloop[MISMATCH + 1][UNDEF + 1][UNDEF + 1][UNDEF + 1][UNDEF + 1];
/* Stability corrections for tetraloops */
energy hairpin_stacking[MISMATCH + 1][UNDEF + 1][UNDEF + 1]; /* Terminal base
							      * stacking
							      * effects in
							      * hairpin loops
							      */
energy loop_stacking[MISMATCH + 1][UNDEF + 1][UNDEF + 1]; /* Terminal base
							   * stacking effects
							   * in interior loops
							   */
int small_loop_size;   /* Number of unpaired bases in smallest loop needed
			* for the energy function to be non-special
			*/

int smallest_hairpin; /* Smallest possible size of a hairpin loop */

int short_stem_length; /* Number of unpaired bases in shortest stem needed
			* for the asymmetry function to satisfy
			* asymmetry(m+1, n+1) = asymmetry(m, n) + f(m + n)
			*/
energy min_stacking; /* The most stability gained by stacking of unpaired
		      * bases in interior loops
		      */
int loop_table_size; /* Size of largest loops for which stabilty terms are
		      * tabulated; for larger loops the logarithmic
		      * extrapolation is used
		      */
/* loopixj is a table of special stability values for i by j interior loops */
energy loop1x1[MISMATCH + 1][MISMATCH + 1][UNDEF + 1][UNDEF + 1];
energy loop1x2[MISMATCH + 1][MISMATCH + 1][UNDEF + 1][UNDEF + 1][UNDEF + 1];
energy loop2x2[MISMATCH + 1][MISMATCH + 1][UNDEF + 1][UNDEF + 1][UNDEF + 1][UNDEF + 1];

//bp basepair(base a, base b);
bp basepair[UNDEF + 1][UNDEF + 1];
void init_bp();
boolean1 equal_energies(energy a, energy b);
energy add_energies(energy a, energy b);
energy times2(energy E);
void printenergy(energy E);
energy eL_noextstackingandsize(int i, int j, int k, int l, sequence s);
energy eL(int i, int j, int k, int l, sequence* s);
energy eLstacking(int i, int j, sequence s);
energy size(int n);
energy ninio(int m, int n);
energy g(int n);
energy eh(int i, int j, sequence s);
#ifdef EXTERIOR
energy he(int i, int j, sequence s);
#endif
energy e(int i, int j, sequence s);
energy dangling3(int i, int j, sequence s);
energy dangling5(int i, int j, sequence s);
energy dangling3and5(int i, int j, sequence s);
void test();

#endif
