/* rnaSlider.c 
 * Authors: Ydo Wexler & Michal Ziv-Ukelson 2008
 * RNA secondary structure prediction of all consecutive sliding windows in a sequence.
 * The algorithm is based on the papers "A Study of Accessible Motifs and RNA Folding Complexity" 
 by Ydo Wexler, Chaya Zilberstein and Michal Ziv-Ukelson (JCB, 2007), 
 and the paper  "A fast algorithm to scan the RNA folding energy landscape of genomes 
 and its application to the analysis of genomic folding asymmetry" 
 by Yair Horesh, Ydo Wexler, Michal Ziv-Ukelson, and Ron Unger.

 Anyone using, modyfying, or redistributing this code is required to reference these works.
 Moreover, the copywrite notice must not be removed with any manipulation / redistribution of the code
 or the software.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE 
 FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
 ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
 EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *              
 */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>
#include <limits.h>
#include <string.h>
#include "rnaSlider.h"
#include "common.h"
#include "energyfiles/energy.h"
#include "energyfiles/energy_read.h"
#include "../rnaSurface.h"

const char retrobase[4] = {'a', 'c', 'g', 'u'};
/* minextension is only used in preinterior (except if compiled with the WTEST
 * directive). To ease initialization it is declared extern instead of static
 * in preinterior.
 */
static int minextension;

/* Tables used in the dynamic programming */
//#define MAX_LENGTH 200000000
#define WEAK_MONGE 1
/*
energy** V;
energy** W;
energy** VBI;
energy** VM;
*/
// -1 is a single structure
// -2 is an unpaired dangling
// -3 is a unfolded subsequence
// anything else is a partition point
int TRACEBACK = 0;
int** tracebackStructure;
char* tracebackChars;

/* candidate lists for weak monge */
/*
int** Wcandidates;
energy** VMcandidates;
int* lastWcand;
int winPoint = 0;
int winSize;

sequence s;
*/
#define UNSAFESTOPRULE 1
#ifdef UNSAFESTOPRULE
int largest_loop = 15;
#endif

double exp_const=0.62;





/* allocate_tables(s): Allocate and initialize tables used in the dynamic
 * programming.
 */
void initialize_tables(sequence s)
{
	int i, j;

	/* Initialize tables */
	for (i=0; i < s.length+1; i++) {
		for (j=0; j < s.length+1; j++) {

			V[i][j] = INFINITEENERGY;
			W[i][j] = INFINITEENERGY;
	
			VBI[i][j] = INFINITEENERGY;
			VM[i][j] = INFINITEENERGY;
		}
	}

}

/*
	initialize experimental paired probability array
*/
void initialize_exp(char* filename,char type,int length)
{
	FILE* fr;
	int i,pos=0,len,response;
	double prob=0;
	if ( (fr=fopen(filename,"r")) == NULL )
	{
		if (type=='p')
			pprob_exist=0;
		else if (type=='u')
			uprob_exist=0;
		else if (type=='r')
			react_exist=0;
		fprintf(stderr,"File with experimental probabilities %s not found",filename);
		exit(1);
	}
	else
	{
		if (type=='p')
		{
			pprob=(double*)(malloc(length*sizeof(double)));
			for (i=0;i<length;i++) pprob[i]=-1;
			pprob_exist=1;
		}
		else if (type=='u')
		{
			uprob=(double*)(malloc(length*sizeof(double)));
			for (i=0;i<length;i++) uprob[i]=-1;
			uprob_exist=1;
		}
		else if (type=='r')
		{
			react=(double*)(malloc(length*sizeof(double)));
			for (i=0;i<length;i++) react[i]=-1;
			react_exist=1;		
		}
	}
	len=0;
	while ( feof(fr)==0 && pos<length )
	{
		response=fscanf(fr,"%d %lf\n",&pos,&prob);
		if ( response!=2 && response!=-1 ){
			fprintf(stderr,"\nError while reading file %s : line does not match 'int double' representation\n",filename);
			exit(2);
		}
		if (response==2){
			len++;
			if ( (prob<0) && (type=='u' || type=='p') )
			{
				fprintf(stderr,"File %s contains incorrect probability/density value:\n %d %lf\n",filename,pos,prob);
				exit(1);
			}
			if (type=='p')
				pprob[pos-1]=prob;
			
			else if (type=='u')
				uprob[pos-1]=prob;
		
			else if (type=='r')
				react[pos-1]=prob;
		}
	}
	fclose(fr);
}

/* return RT*ln(P_i/U_i) */
double pcoef(int i)
{
	int pos=i+winPoint;
	if (pprob[pos]<0 || uprob[pos]<0)
		return   0  ;  
	else return -exp_const*(log(pprob[pos])-log(uprob[pos]))  ;
}

/* return sum of unpaired weights of [i..j] positions */
double ucoef(int i, int j)
{
	int k,pos;
	double sm;
	sm=0;
	for (k=i;k<j+1;k++)
	{
		pos=k+winPoint;
		if (uprob[pos]<0)
			sm += 0 ;
		else 
			sm += 0;//-exp_const*(log(uprob[pos])-log(pprob[pos])) ;

	}
	return sm;
}

/* return experimental weight of paired nt */
double rcoef(int i)
{
	int pos=i+winPoint;
	if (react[pos] > -1 /*-500*/)
		return (2.6*log(react[pos]+1)-0.8);
	else
		return 0;
}

/* free tables */
void free_tables() {

  free(loop);
  free(bulge);
  free(hairpin);
  free(f);
  
  if (pprob_exist==1)
	free(pprob);
  if (uprob_exist==1)
	free(uprob);
  if (react_exist==1)
	free(react);
	
}




energy optEnergy(char* str, int length)
{
  int h, l, c, k;
  int i, j;
  energy minEnergy, tmpEnergy, helpEnergy;
  energy res;
  int maxLoop;
  int curBest;
  double exp_weight,exp_average;
  energy curVM, bestVM;
 // clock_t start_time;
//	double round_time=0;

  /* Input sequence */
	s.length = 0;
	//s.base = mallocate(length*sizeof(base));
  for (k=0; k<length; k++) {

      c = str[k];
      switch(c)
		{
		case 'A':
		case 'a':
		  s.base[s.length] = A;
		  s.length++;
		  break;
		case 'C':
		case 'c':
		  s.base[s.length] = C;
		  s.length++;
		  break;
		case 'G':
		case 'g':
		  s.base[s.length] = G;
		  s.length++;
		  break;
		case 'T':
		case 't':
		case 'U':
		case 'u':
		  s.base[s.length] = U;
		  s.length++;
		  break;
		case 'Y':
		case 'y':
		  s.base[s.length] = UNDEF;
		  s.length++;
		  break;
		case '\n':
		case '\t':
		case ' ':
		case EOF:
		  break;
		default:
		  fprintf(stderr, "Unrecognized character '%c' in input sequence. ; ", c);
		  fprintf(stderr,"winpoint: %d",winPoint);
		  break;
		}
  }

	initialize_tables(s);



	/* start the main loops */
	

	// use the information from the times there was no window


	i = s.length-1;

	for (; i>=0 ;i--) {
		if (WEAK_MONGE == 1) {

			lastWcand[i]=1;
			Wcandidates[i][0]=i;
		}

		for (j=i+4; j<s.length ;j++) {

			// compute VM and keep track of candidates

			/* naive O(n^3) computation */
			if (WEAK_MONGE == 0) {
				
				for (k=i+1; k<j-1 ;k++) {

					tmpEnergy = add_energies(W[i+1][k], W[k+1][j-1]);
					VM[i][j] = mins(VM[i][j], add_energies(tmpEnergy, mloop_a));
				}
			}

			// compute VBI + (es+V) which is a special case of i'-i+j-j'=2
			// VBI(i,j)
			// eL is ebi or es in case i'-i+j-j'=2

			//start_time = clock();
			minEnergy = INFINITEENERGY;

			// h = i'-i  l = j - j'
			if (largest_loop < j-i-1) {
				maxLoop = largest_loop;
			}
			else {
				maxLoop = j-i-1;
			}

		   if (basepair[s.base[i]][s.base[j]] != MISMATCH)
		   {  	
			for (h=1; h<maxLoop ;h++) {
				for (l=1; l<=maxLoop-h ;l++) {
					

					// this is inline efficient function eL()

					if ((basepair[s.base[i]][s.base[j]] == MISMATCH)
						|| (basepair[s.base[i+h]][s.base[j-l]] == MISMATCH)) {
						tmpEnergy = INFINITEENERGY;
					}
					else {
						if ((h>1) && (l>1)) {
							/* Real interior loop */
							if ((h+l > 6) || ((h+l == 6) && (h != 3))) {
							  /* Interior loop of size larger than 4 or 1x3/3x1 loop */

								c = h;
								if (l < c) {
									c = l;
								}
								--c;

								tmpEnergy = abs(h-l) * f[(c < short_stem_length ? c : short_stem_length)];

								if (ninio_max_correction < tmpEnergy) {
									tmpEnergy = ninio_max_correction;
								}


								c = (h + l - 2);
								helpEnergy = c > loop_table_size ? loop[loop_table_size]
									  + (energy)(logparam * log((float) c / (float) loop_table_size))
									  : loop[c];
								

								if ((tmpEnergy >= INFINITEENERGY) || (helpEnergy >= INFINITEENERGY)) {
									tmpEnergy = INFINITEENERGY;
								}
								else {
									tmpEnergy += helpEnergy;
								}
								
								helpEnergy = loop_stacking[basepair[s.base[i]][s.base[j]]]
								[s.base[i + 1]][s.base[j - 1]];

								if ((tmpEnergy >= INFINITEENERGY) || (helpEnergy >= INFINITEENERGY)) {
									tmpEnergy = INFINITEENERGY;
								}
								else {
									tmpEnergy += helpEnergy;
								}

								helpEnergy = loop_stacking[basepair[s.base[j-l]][s.base[i+h]]]
								[s.base[j-l + 1]][s.base[i+h - 1]];
								

								if ((tmpEnergy >= INFINITEENERGY) || (helpEnergy >= INFINITEENERGY)) {
									tmpEnergy = INFINITEENERGY;
								}
								else {
									tmpEnergy += helpEnergy;
								}

							
							}
							else if (h+l == 6) {
							  /* 2x2 loop */
							  tmpEnergy = loop2x2[basepair[s.base[i]][s.base[j]]]
								[basepair[s.base[i+h]][s.base[j-l]]]
								[s.base[i + 1]][s.base[i + 2]][s.base[j - 1]][s.base[j - 2]];
							}
							else if (h+l == 4) {
								  /* 1x1 loop */
								tmpEnergy = loop1x1[basepair[s.base[i]][s.base[j]]]
									[basepair[s.base[i+h]][s.base[j-l]]]
									[s.base[i + 1]][s.base[j - 1]];
							}
							else if (h == 2) {
							  /* 1x2 loop */
							  tmpEnergy = (loop1x2[basepair[s.base[i]][s.base[j]]]
								  [basepair[s.base[i+h]][s.base[j-l]]]
								  [s.base[i + 1]][s.base[j - 1]][s.base[j - 2]]);
							}
							else {
							  /* 2x1 loop */
							  tmpEnergy = (loop1x2[basepair[s.base[j-l]][s.base[i+h]]]
								  [basepair[s.base[j]][s.base[i]]]
								  [s.base[j - 1]][s.base[i + 2]][s.base[i + 1]]);
							}
						}
						else if ((h == 1) && (l == 1)) {
							/* Helix stacking */
							// es
							tmpEnergy = stack[basepair[s.base[i]][s.base[j]]][basepair[s.base[i+h]][s.base[j-l]]];
						}
						else if (h+l == 3) {
							/* Bulge of size 1 */
							tmpEnergy = add_energies(bulge[1],
							   stack[basepair[s.base[i]][s.base[j]]][basepair[s.base[i+h]][s.base[j-l]]]);
						}
						/* Bulge of size larger than 1 */
						else if (h + j > loop_table_size + l + 2) {
						  tmpEnergy = add_energies(bulge[loop_table_size],
						   (energy)(logparam * log((float) (h + l - 2) / (float) loop_table_size)));
						}
						else {
							tmpEnergy = bulge[h + l - 2];
						}
					}

					if ((tmpEnergy < INFINITEENERGY) && (V[i+h][j-l] < INFINITEENERGY)) {

						tmpEnergy +=  V[i+h][j-l];
						
						exp_weight=0;exp_average=0;
						if (pprob_exist==1)
							exp_weight += pcoef(i)+pcoef(j); //add paired information
						if (uprob_exist==1)	
							exp_weight += ucoef(i+1,i+h-1)+ucoef(j-l+1,j-1); //add unpaired information
						if (react_exist==1){
							exp_weight += rcoef(i)+rcoef(j); //add shape-reactivity information			
						}
						
						tmpEnergy+=exp_weight;
						
						if (tmpEnergy < minEnergy) {
							minEnergy =  tmpEnergy;
						}
					}


					// this line was replace (for efficiency) with the above section
				//	minEnergy = mins(minEnergy, add_energies(eL(i, j, i+h, j-l, &s), V[i+h][j-l]));
				}
			}
			}
			VBI[i][j] = minEnergy;

			// compute V
			V[i][j] = mins(VBI[i][j], VM[i][j]);
			
			exp_weight=0;
			if (pprob_exist==1)
				exp_weight += pcoef(i)+pcoef(j);
			if (uprob_exist==1)
				exp_weight += ucoef(i+1,j-1);
			if (react_exist==1)
				exp_weight += rcoef(i)+rcoef(j); 

			V[i][j] = mins(V[i][j], eh(i, j, s)+exp_weight);
			////round_time += ((double)(clock() - start_time))/((double)CLOCKS_PER_SEC);
			// compute W and keep track of candidates

			/* naive O(n^3) computation */
			if (WEAK_MONGE == 0) {

				W[i][j] = mins(W[i+1][j], W[i][j-1]);

				for (k=i; k<j ;k++) {

					if ((W[i][k] < INFINITEENERGY) && (W[k+1][j] < INFINITEENERGY)) {

						tmpEnergy +=  W[i][k] + W[k+1][j];

						if (tmpEnergy < W[i][j]) {
							W[i][j] = tmpEnergy;
						}
					}
					
				}

				W[i][j] = mins(W[i][j], V[i][j]);
			}
			/* fast O(n^2 k) weak monge computation */
			else {

				minEnergy = INFINITEENERGY;
				bestVM = INFINITEENERGY;
				curBest = INT_MAX;

				if (TRACEBACK) {
					tracebackStructure[i][j] = -3;
				}

				// iterate over the existing candidates
				// choose the one that gives the minimum
				for (l=0; l<lastWcand[i] ;l++) {

					k = Wcandidates[i][l];
					
					if ((k-i >= 4) && (j-k-1 >= 4)) {
						tmpEnergy = add_energies(W[i][k], W[k+1][j]);
						curVM = tmpEnergy;
					}
					else if ((k-i < 4) && (j-k-1 < 4)) {
						tmpEnergy = INFINITEENERGY;
						curVM = INFINITEENERGY;
					}
					else if (k-i >= 4) {
						exp_weight=0;
						if (uprob_exist==1)
							exp_weight=ucoef(k+1,j);
						tmpEnergy = W[i][k]+exp_weight;
						curVM = INFINITEENERGY;
					}
					else {
						exp_weight=0;
						if (uprob_exist==1)
							exp_weight=ucoef(i,k);
						tmpEnergy = W[k+1][j]+exp_weight;
						curVM = INFINITEENERGY;
					}

					if (tmpEnergy < minEnergy) {
						minEnergy = tmpEnergy;
						curBest = k;
						
						if (TRACEBACK > 0) {
							tracebackStructure[i][j] = k;
						}
					}

					if (curVM < bestVM) {
						bestVM = curVM;
					}
				}

				// a new candidate is born

				if (TRACEBACK) {
					if (minEnergy > W[i][j-1]) {
						tracebackStructure[i][j] = -2;
					}
				}

				exp_weight=0;
				if (uprob_exist==1)
					exp_weight=ucoef(j,j);
				minEnergy = mins(minEnergy, W[i][j-1]+exp_weight);
				
					
				if (V[i][j] < minEnergy) {

					Wcandidates[i][lastWcand[i]] = j; 
					++lastWcand[i];

					if (TRACEBACK) {
						tracebackStructure[i][j] = -1;
					}
				}

				W[i][j] = mins(minEnergy, V[i][j]);

				if ((i > 0) && (j < s.length-1)) {
					exp_weight=0;
					//if (pprob_exist==1 && !(basepair[s.base[i-1]][s.base[j+1]] == MISMATCH) )
					//	exp_weight=pcoef(i-1)+pcoef(j+1);
					//if (react_exist==1)
					//	exp_weight=rcoef(i-1)+rcoef(j+1);
					VM[i-1][j+1] = add_energies(bestVM, mloop_a)+exp_weight;
					
					/*
					if ( basepair[s.base[i-1]][s.base[j+1]] == MISMATCH ) {
						VM[i-1][j+1] = INFINITEENERGY;
					}
					else{
						VM[i-1][j+1] = add_energies(bestVM, mloop_a)+exp_weight;
					}
					*/
					
				}

			}

		}
	}
	
	if (makeParamCalculation==1) calculateParam();
	res = W[0][s.length - 1];
	//printf("res: %lf\n",res);
	return res;
}


energy extendOptEnergy(char* str, int length)
{
  int h, l, c, k;
  int i, j;
  energy minEnergy, tmpEnergy, helpEnergy;
  energy res;
  int maxLoop;
  int curBest;
  energy curVM, bestVM;

  int newRowIndx;
  int rowIndx;
  int colIndx;
  int kDistI, kDistJ;
  double exp_weight;

  //clock_t start_time;
  //double round_time=0;

  /* Input sequence */
	s.length = 0;
	////s.base = mallocate(length*sizeof(base)); 

  for (k=0; k<length; k++) {

      c = str[k];
      switch(c)
		{
		case 'A':
		case 'a':
		  s.base[s.length] = A;
		  s.length++;
		  break;
		case 'C':
		case 'c':
		  s.base[s.length] = C;
		  s.length++;
		  break;
		case 'G':
		case 'g':
		  s.base[s.length] = G;
		  s.length++;
		  break;
		case 'T':
		case 't':
		case 'U':
		case 'u':
		  s.base[s.length] = U;
		  s.length++;
		  break;
		case 'Y':
		case 'y':
		  s.base[s.length] = UNDEF;
		  s.length++;
		  break;
		case '\n':
		case '\t':
		case ' ':
		case EOF:
		  break;
		default:
		  fprintf(stderr, "Unrecognized character '%c' in input sequence. ; ", c);
		  break;
		}
  }

	/* start the main loops */
	
	

	newRowIndx = (winPoint-1)%winSize;
	i = s.length-1;

	// first compute for the new Row (which is of size 1)
	lastWcand[newRowIndx]=1;
	Wcandidates[newRowIndx][0]=i+winPoint;
	

	// compute for the rest of the rows but only the new column 
	for (; i>=0 ;i--) {

		j=s.length - 1;

		if (i+4<s.length) {
	

			// compute VBI + (es+V) which is a special case of i'-i+j-j'=2
			// VBI(i,j)
			// eL is ebi or es in case i'-i+j-j'=2

			////start_time = clock();
			minEnergy = INFINITEENERGY;

			if (largest_loop < j-i-1) {
				maxLoop = largest_loop;
			}
			else {
				maxLoop = j-i-1;
			}


			for (h=1; h<maxLoop ;h++) {
				for (l=1; l<=maxLoop-h ;l++) {
					

					// this is inline efficient function eL()

					if ((basepair[s.base[i]][s.base[j]] == MISMATCH)
						|| (basepair[s.base[i+h]][s.base[j-l]] == MISMATCH)) {
						tmpEnergy = INFINITEENERGY;
					}
					else {
						if ((h>1) && (l>1)) {
							/* Real interior loop */
							if ((h+l > 6) || ((h+l == 6) && (h != 3))) {
							  /* Interior loop of size larger than 4 or 1x3/3x1 loop */

								c = h;
								if (l < c) {
									c = l;
								}
								--c;

								tmpEnergy = abs(h-l) * f[(c < short_stem_length ? c : short_stem_length)];

								if (ninio_max_correction < tmpEnergy) {
									tmpEnergy = ninio_max_correction;
								}


								c = (h + l - 2);
								helpEnergy = c > loop_table_size ? loop[loop_table_size]
									  + (energy)(logparam * log((float) c / (float) loop_table_size))
									  : loop[c];
								

								if ((tmpEnergy >= INFINITEENERGY) || (helpEnergy >= INFINITEENERGY)) {
									tmpEnergy = INFINITEENERGY;
								}
								else {
									tmpEnergy += helpEnergy;
								}
								
								helpEnergy = loop_stacking[basepair[s.base[i]][s.base[j]]]
								[s.base[i + 1]][s.base[j - 1]];

								if ((tmpEnergy >= INFINITEENERGY) || (helpEnergy >= INFINITEENERGY)) {
									tmpEnergy = INFINITEENERGY;
								}
								else {
									tmpEnergy += helpEnergy;
								}

								helpEnergy = loop_stacking[basepair[s.base[j-l]][s.base[i+h]]]
								[s.base[j-l + 1]][s.base[i+h - 1]];
								

								if ((tmpEnergy >= INFINITEENERGY) || (helpEnergy >= INFINITEENERGY)) {
									tmpEnergy = INFINITEENERGY;
								}
								else {
									tmpEnergy += helpEnergy;
								}
							}
							else if (h+l == 6) {
							  /* 2x2 loop */
							  tmpEnergy = loop2x2[basepair[s.base[i]][s.base[j]]]
								[basepair[s.base[i+h]][s.base[j-l]]]
								[s.base[i + 1]][s.base[i + 2]][s.base[j - 1]][s.base[j - 2]];
							}
							else if (h+l == 4) {
								  /* 1x1 loop */
								tmpEnergy = loop1x1[basepair[s.base[i]][s.base[j]]]
									[basepair[s.base[i+h]][s.base[j-l]]]
									[s.base[i + 1]][s.base[j - 1]];
							}
							else if (h == 2) {
							  /* 1x2 loop */
							  tmpEnergy = (loop1x2[basepair[s.base[i]][s.base[j]]]
								  [basepair[s.base[i+h]][s.base[j-l]]]
								  [s.base[i + 1]][s.base[j - 1]][s.base[j - 2]]);
							}
							else {
							  /* 2x1 loop */
							  tmpEnergy = (loop1x2[basepair[s.base[j-l]][s.base[i+h]]]
								  [basepair[s.base[j]][s.base[i]]]
								  [s.base[j - 1]][s.base[i + 2]][s.base[i + 1]]);
							}
						}
						else if ((h == 1) && (l == 1)) {
							/* Helix stacking */
							// es
							tmpEnergy = stack[basepair[s.base[i]][s.base[j]]][basepair[s.base[i+h]][s.base[j-l]]];
						}
						else if (h+l == 3) {
							/* Bulge of size 1 */
							tmpEnergy = add_energies(bulge[1],
							   stack[basepair[s.base[i]][s.base[j]]][basepair[s.base[i+h]][s.base[j-l]]]);
						}
						/* Bulge of size larger than 1 */
						else if (h + j > loop_table_size + l + 2) {
						  tmpEnergy = add_energies(bulge[loop_table_size],
						   (energy)(logparam * log((float) (h + l - 2) / (float) loop_table_size)));
						}
						else {
							tmpEnergy = bulge[h + l - 2];
						}
					}


					rowIndx = (i+winPoint+h) % winSize;
					colIndx= (j+winPoint-l) % winSize;
					if ((tmpEnergy < INFINITEENERGY) && (V[rowIndx][colIndx] < INFINITEENERGY)) {

						tmpEnergy +=  V[rowIndx][colIndx];
						
						
						if (pprob_exist==1)
							tmpEnergy += pcoef(i)+pcoef(j); //add paired information
						if (uprob_exist==1)	
							tmpEnergy += ucoef(i+1,i+h-1)+ucoef(j-l+1,j-1); //add unpaired information
						if (react_exist==1)
							tmpEnergy += rcoef(i)+rcoef(j); //add shape-reactivity information
						
						
						if (tmpEnergy < minEnergy) {
							minEnergy =  tmpEnergy;
						}
					}


					// this line was replace (for efficiency) with the above section
				//	minEnergy = mins(minEnergy, add_energies(eL(i, j, i+h, j-l, &s), V[i+h][j-l]));
				}
			}

			rowIndx = (i+winPoint) % winSize;
			colIndx= (j+winPoint) % winSize;

			VBI[rowIndx][colIndx] = minEnergy;
			V[rowIndx][colIndx] = mins(VBI[rowIndx][colIndx], VM[rowIndx][colIndx]);

			exp_weight=0;
			
			if (pprob_exist==1)
				exp_weight += pcoef(i)+pcoef(j);
			if (uprob_exist==1)
				exp_weight += ucoef(i+1,j-1);
			if (react_exist==1)
				exp_weight += rcoef(i)+rcoef(j); 
			
			// compute V
			V[rowIndx][colIndx] = mins(V[rowIndx][colIndx], eh(i, j, s)+exp_weight);

			//round_time += ((double)(clock() - start_time))/((double)CLOCKS_PER_SEC);
			// compute W and keep track of candidates

			
			/* fast O(n^2 k) weak monge computation */
			minEnergy = INFINITEENERGY;
			bestVM = INFINITEENERGY;
			curBest = INT_MAX;

			if (TRACEBACK) {
				tracebackStructure[rowIndx][colIndx] = -3;
			}


			// iterate over the existing candidates
			// choose the one that gives the minimum
			for (l=0; l<lastWcand[rowIndx] ;l++) {

				k = (Wcandidates[rowIndx][l] - winPoint) % winSize;
				
				kDistI = k-i;
				if (kDistI < 0) {
					kDistI = winSize - kDistI;
				}

				kDistJ = j-k-1;
				if (kDistJ < 0) {
					kDistJ = winSize - (j-k) - 1;
				}

				
				if ((kDistI >= 4) && (kDistJ >= 4)) {
					tmpEnergy = add_energies(W[rowIndx][(k+winPoint)%winSize], W[(k+winPoint+1)%winSize][colIndx]);
					curVM = tmpEnergy;
				}
				else if ((kDistI < 4) && (kDistJ < 4)) {
					tmpEnergy = INFINITEENERGY;
					curVM = INFINITEENERGY;
				}
				else if (kDistI >= 4) {
					exp_weight=0;
					if (uprob_exist==1)
						exp_weight=ucoef(k+1,j);
					tmpEnergy = W[rowIndx][(k+winPoint)%winSize]+exp_weight;
					curVM = INFINITEENERGY;
				}
				else { 
					exp_weight=0;
					if (uprob_exist==1)
						exp_weight=ucoef(i,k);
					tmpEnergy = W[(k+winPoint+1)%winSize][colIndx]+exp_weight;
					curVM = INFINITEENERGY;
				}

				if (tmpEnergy < minEnergy) {
					minEnergy = tmpEnergy;
					curBest = k; 

					if (TRACEBACK > 0) {
						tracebackStructure[rowIndx][colIndx] = (k+winPoint)%winSize;
					}
				}

				if (curVM < bestVM) {
					bestVM = curVM;
				}
			}

			// a new candidate is born

			
			if (TRACEBACK) {
				if (minEnergy > W[rowIndx][(colIndx+winSize-1)%winSize]) {
					tracebackStructure[rowIndx][colIndx] = -2;
				}
			}

			exp_weight=0;
			if (uprob_exist==1)
				exp_weight=ucoef(j,j);
			minEnergy = mins(minEnergy, W[rowIndx][(colIndx+winSize-1)%winSize]+exp_weight);
				
			if (V[rowIndx][colIndx] < minEnergy) {

				Wcandidates[rowIndx][lastWcand[rowIndx]] = winSize+winPoint-1;
				++lastWcand[rowIndx];

				if (TRACEBACK) {
					tracebackStructure[rowIndx][colIndx] = -1;
				}
			}

			W[rowIndx][colIndx] = mins(minEnergy, V[rowIndx][colIndx]);

			VM[(rowIndx+winSize-1)%winSize][(colIndx+1)%winSize] = add_energies(bestVM, mloop_a);
		}
	}
	if (makeParamCalculation==1) calculateParam();
	res = W[(newRowIndx+1)%winSize][newRowIndx];
//	free(s.base);
	return res;
}

double runSlider(char* str, double* ppaired, double* upaired, int wmax, int wmin, double intersect,char* argv){
	int i,k,length,endPoint,usepar;
	char path[PATH_LEN];
	char* head,*empty;
	double energy;
	winSize=wmax;
	winMin=wmin;
	length=strlen(str);
	TRACEBACK=0;
	/* initialization of energy matrixes */
	V = (float**)malloc((winSize+1)*sizeof(float*));
	W = (float**)malloc((winSize+1)*sizeof(float*));
	VBI = (float**)malloc((winSize+1)*sizeof(float*));
	VM = (float**)malloc((winSize+1)*sizeof(float*));

	Wcandidates = (int**)malloc((winSize+1)*sizeof(int*));
	VMcandidates = (float**)malloc((winSize+1)*sizeof(float*));
	lastWcand = (int*)malloc((winSize+1)*sizeof(int));

	for (k=0; k<(winSize+1) ;k++) {
		V[k] = (float*)malloc((winSize+1)*sizeof(float));
		W[k] = (float*)malloc((winSize+1)*sizeof(float));
		VBI[k] = (float*)malloc((winSize+1)*sizeof(float));
		VM[k] = (float*)malloc((winSize+1)*sizeof(float));

		Wcandidates[k] = (int*)malloc((winSize+1)*sizeof(int));
		VMcandidates[k] = (float*)malloc((winSize+1)*sizeof(float));

	}
	s.base = mallocate(winSize*sizeof(base));

	if ( strcmp(PLATFORM,"win") ==0 )
		GetModuleFileName( NULL, path, PATH_LEN + 1 );
	else
		realpath (argv, path);

	head=(char*)malloc(PATH_LEN*sizeof(char));
	pathHead(path,head);
	read_tables(head);
	free(head);
	/* initialization of experimental data */
	pprob_exist=0;uprob_exist=0;
	
	if (ppaired!=NULL && upaired!=NULL){
		pprob_exist=1;
		uprob_exist=1;
	}
	
	if (pprob_exist==1){
		pprob=(double*)malloc(length*sizeof(double));
		for (i=0;i<length;i++) pprob[i]=ppaired[i];
	}
	
	if (uprob_exist==1){
		uprob=(double*)malloc(length*sizeof(double));
		for (i=0;i<length;i++) uprob[i]=upaired[i];
	}
	react_exist=0;
	winIntersect=intersect;

	initializeEParam(1,empty,&usepar);
	endPoint = length - winSize + 1;
	winPoint=0;
	energy = optEnergy(str, winSize);	
	winPoint = 1;	
	for (k=1; k<endPoint ;k++) {
		energy = extendOptEnergy(str+k, winSize);
		++winPoint;
	}

	for (k=0; k<(winSize+1) ;k++) {		
		free(V[k]);
		free(W[k]);
		free(VBI[k]);
		free(VM[k]);
		free(Wcandidates[k]);
		free(VMcandidates[k]);
	}
	free(V);
	free(W);
	free(VBI);
	free(VM);
	free(Wcandidates);
	free(VMcandidates);
	free(lastWcand);	
	free(s.base);
	free_tables();
	return energy;
}

/* calculate MFE expectation and standard deviation*/
void calculateParam(){
	int len,lenp;
	double en;
	
	for (len=winMin;len<winSize+1;len++)
	{
		/* check whether it is allowable !!! */
		lenp= (int)(len*winIntersect);
		if (winPoint % lenp == 0)
		{
			en=W[winPoint%winSize][(winPoint+len-1)%winSize];
			expParam[len-winMin]+=en;
			sdParam[len-winMin]+=en*en;
			sizeParam[len-winMin]++;
		}
	}
	
}

/* 
	initialize variables to calculate MFE expectation and standard deviation 
	flag: 	0 - do not calculate/use parameters
			1 - calculate parameters
			2 - read parameters from file
*/
void initializeEParam(int flag,char* paramfilename,int* usePar){
	int i,linenum,lineval;
	FILE* fin;
	int len,size;
	double exp, sd;
	*usePar=0;
	if (flag==0)
	{
		makeParamCalculation=0;
	}
	else
	{
		expParam=(double*)malloc((winSize-winMin+1)*sizeof(double));
		sdParam=(double*)malloc((winSize-winMin+1)*sizeof(double));
		sizeParam=(int*)malloc((winSize-winMin+1)*sizeof(int));
		for (i=0;i<winSize-winMin+1;i++)
		{
			expParam[i]=0;
			sdParam[i]=0;
			sizeParam[i]=0;
		}
		if (flag==1)
		{
			makeParamCalculation=1;
		}
		else if (flag==2)
		{
			makeParamCalculation=0;
			if ( (fin=fopen(paramfilename,"r")) == NULL  ) 
			{
				fprintf(stderr,"Wrong filename %s\n",paramfilename);
				fprintf(stderr,"Default tabulated parameters will be used\n");

			}
			else
			{
				*usePar=1;
				linenum=1;
				lineval=3;
				while ( feof(fin) == 0 && lineval==3 )
				{
					lineval=fscanf(fin,"%d %lf %lf",&len,&exp,&sd);
					if ( lineval!=3 && lineval!=-1 )
					{
						fprintf(stderr,"Error in reading file %s:\nproblems with line %d, only %d values were found\n",paramfilename,linenum,lineval);
						fprintf(stderr,"Default tabulated parameters will be used\n");
						*usePar=0;
					}
					else
					{
						expParam[len-winMin]=exp;
						sdParam[len-winMin]=sd;
						sizeParam[len-winMin]=-1;
						//printf("%d %lf %lf\n",len,expParam[len-winMin],sdParam[len-winMin]);
					}
					linenum++;
				}
				fclose(fin);
			}
		}
	}
}

void finishParam(char* paramfilename){
	int len,lenp;
	double en;
	FILE* file;
	if (makeParamCalculation==1)
	{
		for (len=winMin;len<winSize+1;len++)
		{
			expParam[len-winMin]/=sizeParam[len-winMin];
			sdParam[len-winMin]=sqrt( (sdParam[len-winMin] - expParam[len-winMin]*expParam[len-winMin]*sizeParam[len-winMin])/(sizeParam[len-winMin]-1) );
		}
	}
	if ( (file=fopen(paramfilename,"w")) == NULL ) fprintf(stderr,"Wrong filename %s\n",paramfilename);
	else
	{
		for (len=winMin;len<winSize+1;len++)
		{
			fprintf(file,"%d\t%lf\t%lf\t%d\n",len,expParam[len-winMin],sdParam[len-winMin],sizeParam[len-winMin]);
		}
	}
	fclose(file);
}

/* traceback the best structure */
void findStructureTraceback(int length, int startpoint) {

	int i;
	int column = (startpoint+length-1) % winSize;
	int row = startpoint % winSize;

	int localStruct = tracebackStructure[row][column];
	if (localStruct == -2) {
		tracebackChars[column] = '.';
		if (length > 1) {
			findStructureTraceback(length-1, row);
		}
	}
	else if	(localStruct == -1) {
		tracebackChars[row] = '(';
		tracebackChars[column] = ')';
		if (length > 2) {
			int newStart = (startpoint+1) % winSize;
			findStructureTraceback(length-2, newStart);
		}
	}
	else if	(localStruct == -3) {
		for (i=startpoint; i<startpoint+length; i++) {
			tracebackChars[i%winSize] = '.';
		}
	}
	else {
		int length1 = localStruct-row+1;
		if (localStruct < row) {
			length1 += winSize;
		}
		findStructureTraceback(length1, row);
		findStructureTraceback(length-length1, (localStruct+1)%winSize);
	}
}

/* write the structure to a file, by using the traceback */
void writeTraceback(FILE* outp, int length) {

	int i;
	findStructureTraceback(length, winPoint);

	for (i=0; i<length; i++) {
		fprintf(outp, "%c", tracebackChars[(i+winPoint)%winSize]);
	}
	fprintf(outp, "\n");
}

/* The input for the main function is:
- An input filename - the sequences in this file must be in a fasta format: contain A,C,G,T
and every sequence must start with the character ">" and the name of the sequence.

- An output filename.

- An upper bound on hte sequence length - must be an integer larger than the sequence length. 
If a sequence is longer than this parameter, the rest of the sequence is ignored.

- Window size.

- Optionally - the size of the largest interior loop to be considered (set to 15 by default). 
Larger sizes increase the run time. If specified, this parameter must be set to an integer larger than 1.
*/

/*
int main(int argc, char* argv[]) {
	
	int i, k, l, cands=0;
	int ts,t;
	int length;
	int upperBoundSize=-1;///// set as the file size if not set differently
	char* str;
	int iterations=0;
	char ch;
	int maxLength=0;
	double avgLength=0;
	double start_time,end_time,start_t,end_t;
	double energy;
	char* seqName[100];

	printf("Ueaf!!");

	int endPoint;
	start_t=clock();
	
	

	/* for reduce the number of allocations in rna */
	
/*	FILE *inp, *outp;
	char* inFilename;
	char* outFilename;


	for(i=1; i< argc; i+=2) {
		if (argv[i][0] != '-') {
			printf("error in input format. Please refer to the readme file.");
			exit(0);
		}
		switch(argv[i][1]) {
			case 'i': inFilename = argv[i+1]; break;
			case 'o': outFilename = argv[i+1]; break;
			case 'u': upperBoundSize = atoi(argv[i+1]); break;
			case 'w': winSize = atoi(argv[i+1]); break;
			case 't': TRACEBACK = atoi(argv[i+1]); break;
			case 'l': largest_loop = atoi(argv[i+1]); break;
		}
	}
	
	
	if (TRACEBACK) {
		tracebackStructure = (int**)malloc(winSize*sizeof(int*));
		for (ts=0; ts<winSize; ts++) {
			tracebackStructure[ts] = (int*)malloc(winSize*sizeof(int));

			for (t=0; t<4; t++) {
				tracebackStructure[ts][(ts+t)%winSize] = -3;
			}
		}

		tracebackChars = (char*)malloc(winSize*sizeof(char));
	}
	
	V = (float**)malloc((winSize+1)*sizeof(float*));
	W = (float**)malloc((winSize+1)*sizeof(float*));
	VBI = (float**)malloc((winSize+1)*sizeof(float*));
	VM = (float**)malloc((winSize+1)*sizeof(float*));

	Wcandidates = (int**)malloc((winSize+1)*sizeof(int*));
	VMcandidates = (float**)malloc((winSize+1)*sizeof(float*));
	lastWcand = (int*)malloc((winSize+1)*sizeof(int));

	for (k=0; k<(winSize+1) ;k++) {
		V[k] = (float*)malloc((winSize+1)*sizeof(float));
		W[k] = (float*)malloc((winSize+1)*sizeof(float));
		VBI[k] = (float*)malloc((winSize+1)*sizeof(float));
		VM[k] = (float*)malloc((winSize+1)*sizeof(float));

		Wcandidates[k] = (int*)malloc((winSize+1)*sizeof(int));
		VMcandidates[k] = (float*)malloc((winSize+1)*sizeof(float));

	}

	read_tables();


	printf("window size: %d", winSize);

	s.base = mallocate(winSize*sizeof(base));
	

	if( (inp = fopen(inFilename, "r" )) == NULL )	{
		printf("Error in openning input file for read\n");
		exit( 140 );
	}

	if (upperBoundSize==-1)
	{
		fseek (inp, 0L, SEEK_END);
		upperBoundSize = ftell (inp);
		fseek (inp, 0L, SEEK_SET);
	}
	
	str = (char*)malloc(upperBoundSize*sizeof(char));
	
	if( (outp = fopen(outFilename, "w" )) == NULL )	{
		printf("Error in openning input file for write\n");
		exit( 140 );
	}

	ch = fgetc( inp );

	// get sequence name
	if (ch == '>') {
		fscanf(inp, "%s",seqName);
	}

	ch = fgetc( inp );
	while ((ch != '\n') && (feof(inp) == 0)) {
		ch = fgetc( inp );
	}
	length = 0;


	
	for(i=0; feof(inp) == 0;)
	{

		switch (ch) {
			case 'A': 
			case 'a': 
			case 'C': 
			case 'c': 
			case 'G':
			case 'g': 
			case 'T': 
			case 't': ++length;
				
				str[i] = ch;++i;break;
			case '>':  
				
				endPoint = length - winSize + 1;

				//fprintf(outp, "seq %d\t %s\t\n", iterations, seqName);
				fprintf(outp, ">%s\t\n", seqName);

				energy = optEnergy(str, winSize);
				fprintf(outp, "0\t%lf\n", energy);
////				
//printf("0\t%lf\n", energy);	
////				
				if (TRACEBACK) {
					writeTraceback(outp, winSize);
				}
				winPoint = 1;
				

				for (k=1; k<endPoint ;k++) {
					
					energy = extendOptEnergy(str+k, winSize);
					fprintf(outp, "%d\t%lf\n", k, energy);
////
//printf("%d\t%lf\n", k, energy);					
////					
					if (TRACEBACK) {
						writeTraceback(outp, winSize);
					}
					++winPoint;
				}

				//fprintf(outp, "\n\n\n");

				
				if (length > maxLength) maxLength = length;
				avgLength += length;
				length=0;
				i=0;

				fscanf(inp, "%s", seqName);
				
				while ((ch != '\n') && (feof(inp) == 0)) {
					ch = fgetc( inp );
				}
				++iterations;
				printf("%s seq number %d\n", seqName, iterations);	
		}
		
		ch = fgetc( inp );
	}
	
	

	if (length > 0) {
		if (length > maxLength) maxLength = length;
		avgLength += length;
		++iterations;
		//printf("seq number %d\n", iterations);
		start_time = clock();

		endPoint = length - winSize + 1;

		fprintf(outp, ">%s\t\n", seqName);

		energy = optEnergy(str, winSize);
		fprintf(outp, "0\t%lf\n", energy);
////				
//printf("0\t%lf\n", energy);	
////		
		if (TRACEBACK) {
			writeTraceback(outp, winSize);
		}
		winPoint = 1;
		
		

		for (k=1; k<endPoint ;k++) {
			energy = extendOptEnergy(str+k, winSize);
			fprintf(outp, "%d\t%lf\n", k, energy);
////		
//printf("%d\t%lf\n", k, energy);			
////			
			if (TRACEBACK) {
				writeTraceback(outp, winSize);
			}
			++winPoint;
		}
	}

	

	avgLength /= iterations;

	//fprintf(outp, "max seq length: %d\n", maxLength);
	//fprintf(outp, "average seq length: %lf\n", avgLength);
	end_t=clock();
	printf("time %lf\n",(end_t-start_t)/CLOCKS_PER_SEC);
	
	
	fclose(outp);
	printf("finished");
	
	free_tables();

	for (k=0; k<(winSize+1) ;k++) {
		free(V[k]);
		free(W[k]);
		free(VBI[k]);
		free(VM[k]);

		free(Wcandidates[k]);
		free(VMcandidates[k]);
	}

	free(V);
	free(W);
	free(VBI);
	free(VM);

	free(Wcandidates);
	free(VMcandidates);
	free(lastWcand);
	
	free(str);
	free(s.base);

	return 0;
}*/


void printWV(int n) {

	int i, j;
	int i1,j1;
	FILE *matOut;
	char *name=(char*)malloc(100*sizeof(char));
	sprintf(name,"matrixs_%d.txt",n);
	if( (matOut = fopen(/*"matrixs.txt"*/name, "w" )) == NULL )	{
		printf("Error in openning matrixs file for write\n");
		exit( 140 );
	}

	fprintf(matOut, "W:\n\n");
	for (i=0; i<s.length ;i++) {
		for (j=0; j<s.length ;j++) {
			i1=(i+winPoint)%winSize;
			j1=(j+winPoint)%winSize;
			if (W(i1, j1) > 1000) {	// it's infinite energy
				fprintf(matOut, "inf\t");
			}
			else {
				fprintf(matOut, "%.2lf\t", W(i1, j1));
			}
		}
		
		fprintf(matOut, "\n");
	}

	fprintf(matOut, "\n\n\nV:\n\n");
	for (i=0; i<s.length ;i++) {
		for (j=0; j<s.length ;j++) {

			if (V(i, j) > 1000) {	// it's infinite energy
				fprintf(matOut, "inf\t");
			}
			else {
				fprintf(matOut, "%.2lf\t", V(i, j));
			}
		}
		
		fprintf(matOut, "\n");
	}

	fprintf(matOut, "\n\n\nVBI:\n\n");
	for (i=0; i<s.length ;i++) {
		for (j=0; j<s.length ;j++) {

			if (VBI[i][j] > 1000) {	// it's infinite energy
				fprintf(matOut, "inf\t");
			}
			else {
				fprintf(matOut, "%.2lf\t", VBI[i][j]);
			}
		}
		
		fprintf(matOut, "\n");
	}

	fprintf(matOut, "\n\n\nVM:\n\n");
	for (i=0; i<s.length ;i++) {
		for (j=0; j<s.length ;j++) {

			if (VM[i][j] > 1000) {	// it's infinite energy
				fprintf(matOut, "inf\t");
			}
			else {
				fprintf(matOut, "%.2lf\t", VM[i][j]);
			}
		}
		
		fprintf(matOut, "\n");
	}

	fclose(matOut);

//	free_tables();
  

}
